/**
 * @file MinosAcqBackCCfg.cpp
 * @date 31 mai 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "MinosAcqBackCCfg.h"
#include "MinosDataValidator.h"
#include "MinosEventBuilder.h"
#include "MinosEventTranscoder.h"
#include "MinosTcpEventSender.h"
#include <mdcs/cfg/CCfgConfig.h>
#include <CCfg/CConfig.h>
#include <utl/compat/ArrayHelper.hpp>
#include <utl/Logging.h>

namespace minos
{
using namespace mdcs;

namespace acq
{
namespace back
{
namespace array = ::utl::array;

template<>
void MinosDataValidator::Params::init(Params& /*params*/, const CCfg::CConfig& /*subconfig*/)
{
}

template<>
void MinosEventTranscoder::Params::init(Params& /*params*/, const CCfg::CConfig& /*subconfig*/)
{
}

template<>
void MinosEventBuilder::Params::init(Params& params, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	params.maxSrcEventSize_B = config("sourceEventMaxSize").getIntValue() * 1024;
	params.bufferSize_B      = config("sourceEventBufferSize").getIntValue() * 1024;
	for(size_t i = 0; i < array::size(params.activeSources); ++i)
	{
		params.activeSources[i] = config("Source", i)("isActive");
	}
}

template<>
void MinosTcpEventSender::Params::init(Params& params, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	params.target.fromString(config("socketAddr"));
	params.timeout_ms      = config("timeout");
}

MinosAcqBackCCfg::MinosAcqBackCCfg()
{
	cfg::Configurator::instance()
		.add( &MinosTcpEventSender::Params::init<CCfg::CConfig>)
		.add(   &MinosEventBuilder::Params::init<CCfg::CConfig>)
		.add(&MinosEventTranscoder::Params::init<CCfg::CConfig>)
		.add(  &MinosDataValidator::Params::init<CCfg::CConfig>)
		;
}

} /* namespace back */
} /* namespace acq */
} /* namespace minos */
