/**
 * @file MinosDataValidator.cpp
 * @date 20 juin 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "MinosDataValidator.h"
#include <minos/fem/FrameTypes.h>
#include <utl/io/InplaceParser.ipp>
#include <utl/Logging.h>

namespace minos
{
namespace acq
{
namespace back
{
using namespace utl::io;
using namespace mdcs::acq;
using namespace mdcs::utl;
using namespace fem;

// =============================================================================
// minos::acq::back::MinosDataValidator::Params class methods body
// -----------------------------------------------------------------------------

MinosDataValidator::Params::~Params()
{
}

// =============================================================================
// minos::acq::back::MinosDataValidator::Setup class methods body
// -----------------------------------------------------------------------------

MinosDataValidator::Setup::Setup(ApplicationState& state, const Params& params)
	: BaseSetupData<Policy>(state, params)
{
}

// =============================================================================
// minos::acq::back::MinosDataValidator::RunImpl class methods body
// -----------------------------------------------------------------------------

MinosDataValidator::RunImpl::RunImpl(ApplicationState& state, Setup& setup, const RunInfo& runInfo)
	: BaseRunImpl<Policy>(state, setup, runInfo)
{
}

MinosDataValidator::RunImpl::~RunImpl()
{
}

void MinosDataValidator::RunImpl::process(const uint8_t* buf, uint32_t nBytes)
{
//	std::cout << std::hex;
//	for(int i = 0; i < 8; ++i)
//	{
//		std::cout << *(reinterpret_cast<const uint32_t*>(buf) + i) << ' ';
//	}
//	std::cout << std::dec << std::endl;

	InplaceParser parser(buffer(buf, nBytes));
	ValueRef<EventHeader> header;
	if(!parser.parse_1(header))
	{
		LOG_WARN() << "invalid event header for event of size: " << nBytes << '!';
		return;
	}
	parser.ignore<uint8_t>(nBytes - sizeof(EventHeader) - sizeof(EventEpilogue));

	ValueRef<EventEpilogue> epilogue;
	if(!parser.parse_1(epilogue))
	{
		LOG_WARN() << "invalid event epilogue for event #" << header->eventId << '!';
		return;
	}

	if(nBytes != epilogue->eventSize())
	{
		LOG_WARN() << "invalid recorded event size for event #" << header->eventId << "! received data size=" << nBytes << " event size=" << epilogue->eventSize();
		return;
	}
	LOG_INFO() << "event #" << header->eventId << " of size: " << nBytes << " validated";
	nextStageProcess(buf, nBytes);
}

} /* namespace back */
} /* namespace acq */
} /* namespace minos */
