/**
 * @file FemAcquisitionBuffer.h
 * @date 15 mai 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef minos_acq_front_FemAcquisitionBuffer_h_INCLUDED
#define minos_acq_front_FemAcquisitionBuffer_h_INCLUDED

#include "FemDataProducer.h"
#include <mdcs/acq/front/AbstractInputBuffer.h>
#include <mdcs/utl/net/Socket.h>
#include <mdcs/utl/ContiguousCircularBuffer.h>
#include <boost/scoped_array.hpp>
#include <unistd.h>

namespace minos
{
namespace fem { class FeminosController; }

namespace acq
{
using namespace mdcs::acq;

namespace front
{

class FemAcquisitionBuffer : public AbstractInputBuffer
{
public:
	FemAcquisitionBuffer(const FemDataProducer::Params::FlowControl& params, fem::FeminosController& ctrl);
	~FemAcquisitionBuffer();

	Handle read(uint32_t maxSize_B, uint32_t maxWait_ms, bool allowSplitFrame);
	void interruptRead();

private:
	size_t receiveFrame(mdcs::utl::net::Milliseconds& recvTimeout);

	void updateThroughputStats(size_t len);

	void handleNonDataFrame(const ::utl::io::ConstBuffer& buf);

	class FeminosReceiver;

	struct EventChunk
	{
		EventChunk(const void* begin);
		void setChunkEnd(const void* end);

		const void* segment;
		uint32_t size;
		bool eventEnd;
	};

	class Iterator : public AbstractInputBuffer::Iterator
	{
	public:
		Iterator(FemAcquisitionBuffer& stream, Status status, size_t bytesRead);
		~Iterator();

		Status getStatus() const;
		bool hasNext() const;
		::utl::io::ConstBuffer next(bool& frameEnd);

	private:
		void dispose();

		FemAcquisitionBuffer& stream_;
		const Status status_;
		const size_t bytesRead_;
		std::vector<EventChunk>::const_iterator currentChunk;
	};

	class FlowManager
	{
	public:
		FlowManager(const FemDataProducer::Params::FlowControl& params, fem::FeminosController& feminos);
		void sendCredits();
		void adjustCredits(size_t lostCount);

	private:
		static const int CONSECUTIVE_MULTIPLIER = 200;
		static const int MISSED_MAX = 10;

		const FemDataProducer::Params::FlowControl& params;
		fem::FeminosController& feminos;
		int credits;
		int creditIncrement;
		int creditLowThreshold;
		int thresholdRaiseIncr;
		int safeThreshold;
		int unsafeThreshold;
		int consecutiveFramesNeeded;
		int missedFramesNeeded;
	};

	friend class Iterator;

	fem::FeminosController& feminos;
	FlowManager flowManager;
	boost::scoped_array<uint8_t> datablock;
	mdcs::utl::ContiguousCircularBuffer frameBuffer;
	std::vector<EventChunk> chunks;
	fem::EventEpilogue missingEpilogueBuffer;
	uint8_t iterator[sizeof(Iterator)];

	::utl::io::InplaceParser parser;
	bool stopping;
	uint8_t lastSendCount;
	size_t lastEventSize;
	size_t lastFrameSize;
	size_t avgFrameSize;

	timespec lastTime;
	size_t sizeDataExtracted;
};

} /* namespace front */
} /* namespace acq */
} /* namespace minos */

#endif /* minos_acq_front_FemAcquisitionBuffer_h_INCLUDED */
