/* =============================================================================
 * AsicChannelID.h
 * -----------------------------------------------------------------------------
 * class minos::fem::AsicID
 * class minos::fem::AsicRange
 * class minos::fem::ChannelID
 * class minos::fem::ChannelRange
 * Created on: 23 janv. 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This software is part of the DaqMinos project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef minos_fem_AsicChannelID_h_INCLUDED
#define minos_fem_AsicChannelID_h_INCLUDED

#include <ostream>
#include <climits>

namespace minos
{
namespace fem
{

typedef unsigned short AsicNum;
typedef unsigned short ChannelNum;

enum AsicInfo
{
	AsicCount = 4
};

enum AsicSpecialID
{
	AllAsics = USHRT_MAX
};

enum ChannelSpecialID
{
	AllChannels = USHRT_MAX
};

class AsicID
{
public:
	AsicID() : value() {}
	AsicID(AsicNum val) : value(val) {}

	friend std::ostream& operator<<(std::ostream& s, const AsicID& r);

private:
	AsicID(AsicSpecialID val) : value(val) {}
	AsicNum value;
};

inline std::ostream& operator<<(std::ostream& s, const AsicID& r)
{
	s << r.value;
	return s;
}

class AsicRange
{
public:
	AsicRange(AsicNum val) : first(val), last(val) {}
	AsicRange(AsicSpecialID val) : first(val), last(val) {}
	AsicRange(AsicNum first, AsicNum last) : first(first), last(last) {}

	friend std::ostream& operator<<(std::ostream& s, const AsicRange& r);

private:
	AsicRange(ChannelSpecialID val);
	AsicNum first;
	AsicNum last;
};

inline std::ostream& operator<<(std::ostream& s, const AsicRange& r)
{
	if(r.first == AllAsics)
	{
		s << '*';
	}
	else if(r.first == r.last)
	{
		s << r.first;
	}
	else
	{
		s << r.first << ':' << r.last;
	}
	return s;
}

class ChannelID
{
public:
	ChannelID() : value() {}
	ChannelID(ChannelNum val) : value(val) {}

	friend std::ostream& operator<<(std::ostream& s, const ChannelID& r);

private:
	ChannelID(ChannelSpecialID val) : value(val) {}
	ChannelNum value;
};

inline std::ostream& operator<<(std::ostream& s, const ChannelID& c)
{
	s << c.value;
	return s;
}

class ChannelRange
{
public:
	ChannelRange(ChannelNum val) : first(val), last(val) {}
	ChannelRange(ChannelSpecialID val) : first(val), last(val) {}
	ChannelRange(ChannelNum first, ChannelNum last) : first(first), last(last) {}

	friend std::ostream& operator<<(std::ostream& s, const ChannelRange& r);

private:
	ChannelRange(AsicSpecialID val);
	ChannelNum first;
	ChannelNum last;
};

inline std::ostream& operator<<(std::ostream& s, const ChannelRange& c)
{
	if(c.first == AllChannels)
	{
		s << '*';
	}
	else if(c.first == c.last)
	{
		s << c.first;
	}
	else
	{
		s << c.first << ':' << c.last;
	}
	return s;
}


} /* namespace fem */
} /* namespace minos */


#endif /* minos_fem_AsicChannelID_h_INCLUDED */
