/**
 * @file BuiltEvent.h
 * @date 18 sept. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef minos_fem_BuiltEvent_h_INCLUDED
#define minos_fem_BuiltEvent_h_INCLUDED

#include "FrameTypes.h"
#include <istream>

namespace minos
{
namespace fem
{

class BuiltEvent
{
public:
	class UntilSize;
	class UntilId;
	class Visitor;
	class BinaryToTextVisitor;

	static void read(std::istream& stream, Visitor& visitor);

private:
	class Parser;
};

class BuiltEvent::Visitor
{
public:
	virtual ~Visitor();
	virtual void onBuiltEventBegin(const BuiltEventHeader& header) = 0;
	virtual void onBuiltEventEnd(const BuiltEventFooter& footer) = 0;
	virtual void onEventBegin(const EventHeader& header) = 0;
	virtual void onEventEnd() = 0;
	virtual void onHitChannelCount(const HitChannelCount& hitCount) = 0;
	virtual void onLastCellRead(const LastCellRead& lastCellRead) = 0;
	virtual void onChannelHitBegin(const ChannelHitIndex& hitIndex) = 0;
	virtual void onChannelHitEnd() = 0;
	virtual void onWaveformBegin(const TimeBinIndex* binIndex) = 0;
	virtual void onWaveformEnd() = 0;
	virtual void onSample(const AdcSample& sample) = 0;
};

class BuiltEvent::BinaryToTextVisitor : public Visitor
{
public:
	BinaryToTextVisitor(std::ostream& s);
	void onBuiltEventBegin(const BuiltEventHeader& header);
	void onBuiltEventEnd(const BuiltEventFooter& footer);
	void onEventBegin(const EventHeader& header);
	void onEventEnd();
	void onHitChannelCount(const HitChannelCount& hitCount);
	void onLastCellRead(const LastCellRead& lastCellRead);
	void onChannelHitBegin(const ChannelHitIndex& hitIndex);
	void onChannelHitEnd();
	void onWaveformBegin(const TimeBinIndex* binIndex);
	void onWaveformEnd();
	void onSample(const AdcSample& sample);
private:
	void indent();
	std::ostream& stream;
	int indentLevel;
	int sampleCount;
	int waveformCount;
	int hitCount;
	int eventCount;
};

class BuiltEvent::UntilSize : public ::utl::io::InplaceParser
{
public:
	UntilSize(const ::utl::io::ConstBuffer& buf);

	uint32_t size() const   { return builtEvtHeader->size; }

private:
	ValueRef<BuiltEventHeader> builtEvtHeader;
};

class BuiltEvent::UntilId : public UntilSize
{
public:
	UntilId(const ::utl::io::ConstBuffer& buf);

	uint64_t timestamp() const { return header->timestamp; }
	uint32_t eventId() const   { return header->eventId; }
	uint8_t  type() const      { return header->type; }

private:
	ValueRef<EventHeader> header;
};

} /* namespace fem */
} /* namespace minos */

#endif /* minos_fem_BuiltEvent_h_INCLUDED */
