/* =============================================================================
 * FeminosResult.h
 * -----------------------------------------------------------------------------
 * class ::
 * Created on: 23 janv. 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This software is part of the DaqMinos project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef minos_fem_FeminosResult_h_INCLUDED
#define minos_fem_FeminosResult_h_INCLUDED

#include "FrameTypes.h"
#include "AsicChannelID.h"
#include <boost/shared_ptr.hpp>
#include <vector>

namespace minos
{
namespace fem
{

class FeminosController;

class ReplyBuffer
{
public:
	std::string buffer;
	size_t length;
};

typedef boost::shared_ptr<ReplyBuffer> ReplyBufferPtr;

class FeminosResult
{
public:
	void setBuffer(const ReplyBufferPtr& b) { buffer_ = b; }
	utl::io::ConstBuffer buffer() const;

private:
	ReplyBufferPtr buffer_;
};

class PedestalSummaryResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ArrayRef<PedestalSummary>& pedestalSummary() const { return pedestalSummary_; }
private:
	ArrayRef<PedestalSummary> pedestalSummary_;
};

class PedestalStatsResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ValueRef<HistoChannel>&  histoChannel()  const { return histoChannel_; }
	const ValueRef<PedestalStats>& pedestalStats() const { return pedestalStats_; }

private:
	ValueRef<HistoChannel> histoChannel_;
	ValueRef<PedestalStats> pedestalStats_;
};

class PedestalHistoResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ValueRef<HistoChannel>&     histoChannel()    const { return histoChannel_; }
	const ArrayRef<PedestalHistoBin>& pedestalHisto()   const { return pedestalHisto_; }

private:
	ValueRef<HistoChannel>     histoChannel_;
	ArrayRef<PedestalHistoBin> pedestalHisto_;
};

class DeadtimeHistoResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ValueRef<DeadtimeStats>&    deadtimeStats()   const { return deadtimeStats_; }
	const ArrayRef<DeadtimeHistoBin>& deadtimeHisto()   const { return deadtimeHisto_; }
private:
	ValueRef<DeadtimeStats>    deadtimeStats_;
	ArrayRef<DeadtimeHistoBin> deadtimeHisto_;
};

class SHistoResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ValueRef<SHisto>& sHisto() const { return sHisto_; }
private:
	ValueRef<SHisto> sHisto_;
};

class HitCountHistoResult : public FeminosResult
{
public:
	friend class FeminosController;

	const ValueRef<HitCountHisto>& hitCountHisto() const { return hitCountHisto_; }
private:
	ValueRef<HitCountHisto> hitCountHisto_;
};

class ListResult : public FeminosResult
{
public:
	friend class FeminosController;

	const std::vector<PedThrList>& pedestalList() const { return lists_; }
private:
	std::vector<PedThrList> lists_;
};

} /* namespace fem */
} /* namespace minos */

#endif /* minos_fem_FeminosResult_h_INCLUDED */
