/* =============================================================================
 * Frame.h
 * -----------------------------------------------------------------------------
 * class minos::fem::Frame
 * Created on: 13 déc. 2012 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This software is part of the DaqMinos project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef minos_fem_Frame_h_INCLUDED
#define minos_fem_Frame_h_INCLUDED

#include "FrameTypes.h"
#include <utl/io/Buffer.hpp>
#include <boost/integer/static_min_max.hpp>
#include <typeinfo>

namespace minos
{
namespace fem
{

class MinosFrame
{
public:
	virtual ~MinosFrame() {}
};
class ConfigFrame : public MinosFrame
{
public:
	friend class ::utl::io::InplaceParser;

	const ConfigFrameStart&       frameStart()  const { return frameStart_; }
	const ValueRef<AsciiMessage>& messageInfo() const { return msgInfo_; }
	const ArrayRef<char>          message()     const { return message_; }
private:
	ValueRef<ConfigFrameStart> frameStart_;
	ValueRef<AsciiMessage>     msgInfo_;
	ArrayRef<char>             message_;
};
class MonitFrame : public MinosFrame
{
public:
	friend class ::utl::io::InplaceParser;

	const MonitFrameStart&            frameStart()      const { return frameStart_; }
	const ValueRef<HistoChannel>&     histoChannel()    const { return histoChannel_; }
	const ArrayRef<PedestalHistoBin>& pedestalHisto()   const { return pedestalHisto_; }
	const ValueRef<PedestalStats>&    pedestalStats()   const { return pedestalStats_; }
	const ArrayRef<PedestalSummary>&  pedestalSummary() const { return pedestalSummary_; }
	const ValueRef<DeadtimeStats>&    deadtimeStats()   const { return deadtimeStats_; }
	const ArrayRef<DeadtimeHistoBin>& deadtimeHisto()   const { return deadtimeHisto_; }
	const ValueRef<HitCountHisto>&    hitCountHisto()   const { return hitCountHisto_; }
	const ValueRef<SHisto>&           sHisto()          const { return sHisto_; }
	const ValueRef<CommandStats>&     commandStats()    const { return cmdStats_; }
	const std::vector<PedThrList>&    pedestalList()    const { return lists_; }
private:
	ValueRef<MonitFrameStart>  frameStart_;
	ValueRef<HistoChannel>     histoChannel_;
	ValueRef<PedestalStats>    pedestalStats_;
	ArrayRef<PedestalSummary>  pedestalSummary_;
	ValueRef<DeadtimeStats>    deadtimeStats_;
	ArrayRef<DeadtimeHistoBin> deadtimeHisto_;
	ArrayRef<PedestalHistoBin> pedestalHisto_;
	ValueRef<HitCountHisto>    hitCountHisto_;
	ValueRef<SHisto>           sHisto_;
	ValueRef<CommandStats>     cmdStats_;
	std::vector<PedThrList>    lists_;
};
class DataFrame : public MinosFrame
{
public:
	friend class ::utl::io::InplaceParser;

	const DataFrameStart&        frameStart()   const { return frameStart_; }
	const ValueRef<DataPreamble> dataPreamble() const { return dataPreamble_; }
	const std::vector<Event>&    events()       const { return events_; }

private:
	ValueRef<DataFrameStart> frameStart_;
	ValueRef<DataPreamble>   dataPreamble_;
	std::vector<Event>       events_;
};

class Frame
{
public:
	Frame();
	~Frame();

	bool parse(const ::utl::io::ConstBuffer& buf);

	const MinosFrame& frame() const { return *frame_; }
	template<class F> const F& frame() const { return dynamic_cast<const F&>(frame()); }
	template<class F> bool isFrame() const { return typeid(frame()) == typeid(F); }

private:
	template<class F>
	F& createFrame();

	MinosFrame* const frame_;
	uint8_t frameData_[
		boost::static_unsigned_max<boost::static_unsigned_max<
			sizeof(ConfigFrame),
			sizeof(DataFrame)>::value,
			sizeof(MonitFrame)>::value
	];
};

} /* namespace fem */
} /* namespace minos */

#endif /* minos_fem_Frame_h_INCLUDED */
