/* =============================================================================
 * FrameTypes.cpp
 * -----------------------------------------------------------------------------
 * Created on: 21 déc. 2012 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This software is part of the DaqMinos project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include "FrameTypes.h"
#include <utl/BitFieldHelper.hpp>
#include <utl/Rounding.hpp>

namespace minos
{
namespace fem
{

// =============================================================================
// minos::fem::AsciiMessage class methods body
// -----------------------------------------------------------------------------

unsigned AsciiMessage::totalLength() const
{
	unsigned len = messageLength + 1; // adding 1 for the null character
	return utl::align<utl::RoundUp>(len, 2u); // aligning on the upper even number.
}

// =============================================================================
// minos::fem::PedestalSummary class methods body
// -----------------------------------------------------------------------------

HistoChannel::HistoChannel()
	: channel(0), chip(0), card(0), prefix(0)
{
}

// =============================================================================
// minos::fem::PedestalSummary class methods body
// -----------------------------------------------------------------------------

PedestalSummary::PedestalSummary()
	: channel(), prefix(0), mean(0), stdev(0)
{
}

// =============================================================================
// minos::fem::ListHeader class methods body
// -----------------------------------------------------------------------------

unsigned ListHeader::listLength() const
{
	return isAfter ? AfterChannelCount : AgetChannelCount;
}

// =============================================================================
// minos::fem::EventEpilogue class methods body
// -----------------------------------------------------------------------------

uint32_t EventEpilogue::eventSize() const
{
	return static_cast<uint32_t>(eventSizeMsb) << 16 | eventSizeLsw;
}

void EventEpilogue::setEventSize(uint32_t size)
{
	eventSizeLsw = utl::BitFieldHelper<uint32_t>::getField<0, 16>(size);
	eventSizeMsb = utl::BitFieldHelper<uint32_t>::getField<16, 4>(size);
}

} /* namespace fem */
} /* namespace minos */
