/**
 * @file TcmController.cpp
 * @date 8 août 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "TcmController.h"
#include <cstdio>

namespace minos
{
namespace fem
{

TcmController::TcmController(const utl::net::SocketAddress& endpoint)
	: BoardController(endpoint)
{
}

TcmController::~TcmController()
{
}

uint16_t TcmController::triggerDelay(TcmTrigger trigger)
{
	FeminosCommand cmd;
	sendCommand(cmd) << "trig_delay " << trigger;
	checkConfigFrame(cmd);
	FemNum tcmNum = -1;
	RegisterID regNum = -1;
	RegValue regValue = -1;
	RegValue delay = -1;
	if(sscanf(cmd.frame<ConfigFrame>().message().get(),
			"0 Tcm(%u) Reg(%u) = %x (%u) Trig_Delay: %x",
			&tcmNum, &regNum, &regValue, &regValue, &delay) != 5)
	{
		throw CommandBadReply(cmd, UnexpectedMessageFormat);
	}
	if(delay == 0xFFFFFFFF)
	{
		throw CommandBadReply(cmd, InvalidResult);
	}
	return delay;
}

void TcmController::triggerDelay(TcmTrigger trigger, uint16_t delay)
{
	FeminosCommand cmd;
	sendCommand(cmd) << "trig_delay " << trigger << ' ' << delay;
	checkConfigFrame(cmd);
}

void TcmController::msgStats_clear()
{
	FeminosCommand cmd;
	sendCommand(cmd) << "port * clr";
	checkConfigFrame(cmd);
}

RegValue TcmController::msgStats_rxCount(int port)
{
	FeminosCommand cmd;
	sendCommand(cmd) << "port " << port << " rx";
	checkConfigFrame(cmd);
	FemNum tcmNum = -1;
	RegisterID portNum = -1;
	RegValue count = -1;
	if(sscanf(cmd.frame<ConfigFrame>().message().get(),
			"0 Tcm(%u) Port(%u) RX count: %u",
			&tcmNum, &portNum, &count) != 3)
	{
		throw CommandBadReply(cmd, UnexpectedMessageFormat);
	}
	if(count == 0xFFFFFFFF)
	{
		throw CommandBadReply(cmd, InvalidResult);
	}
	return count;
}

RegValue TcmController::msgStats_errCount(int port)
{
	FeminosCommand cmd;
	sendCommand(cmd) << "port " << port << " err";
	checkConfigFrame(cmd);
	FemNum tcmNum = -1;
	RegisterID portNum = -1;
	RegValue count = -1;
	if(sscanf(cmd.frame<ConfigFrame>().message().get(),
			"0 Tcm(%u) Port(%u) RX Error count: %u",
			&tcmNum, &portNum, &count) != 3)
	{
		throw CommandBadReply(cmd, UnexpectedMessageFormat);
	}
	if(count == 0xFFFFFFFF)
	{
		throw CommandBadReply(cmd, InvalidResult);
	}
	return count;
}

RegValue TcmController::rxEventCount()
{
	return register_read(26);
}

RegValue TcmController::txEventCount()
{
	return register_read(27);
}

RegValue TcmController::feminosEnabled()
{
	return register_read(16);
}

RegValue TcmController::feminosDetected()
{
	return register_read(17);
}

RegValue TcmController::feminosSampling()
{
	return register_read(18);
}

RegValue TcmController::feminosBusy()
{
	return register_read(19);
}

} /* namespace fem */
} /* namespace minos */
