/**
 * @file TcmController.h
 * @date 8 août 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef minos_fem_TcmController_h_INCLUDED
#define minos_fem_TcmController_h_INCLUDED

#include "BoardController.h"

namespace minos
{
namespace fem
{

enum TcmTrigger
{
	TcmTriggerAuto = 0,
	TcmTriggerNIM,
	TcmTriggerLVDS,
	TcmTriggerTTL,
	TcmTriggerCount
};

class TcmController : public BoardController
{
public:
	TcmController(const utl::net::SocketAddress& endpoint);
	~TcmController();

	uint16_t triggerDelay(TcmTrigger trigger);
	void triggerDelay(TcmTrigger trigger, uint16_t delay);

	void msgStats_clear();
	RegValue msgStats_rxCount(int port);
	RegValue msgStats_errCount(int port);

	RegValue rxEventCount();
	RegValue txEventCount();

	RegValue feminosEnabled();
	RegValue feminosDetected();
	RegValue feminosSampling();
	RegValue feminosBusy();
};

// =============================================================================
// Registers definition
// -----------------------------------------------------------------------------

struct TcmRxErrCounterReg
{
	TcmRxErrCounterReg() {}
	TcmRxErrCounterReg(uint32_t r) : value(r) {}

	union
	{
		uint32_t value;
		struct
		{
			uint8_t rx_cnt;
			uint8_t err_cnt;
		} counter[2];
	};
};

struct TcmSwTrigReg
{
	static const int ID = 20;
	TcmSwTrigReg() {}
	TcmSwTrigReg(uint32_t r) : value(r) {}

	union
	{
		uint32_t value;
		struct
		{
			uint32_t ev_type          : 2;
			uint32_t clr_tstamp       : 1;
			uint32_t clr_evcnt        : 1;
			uint32_t sca_stop         : 1;
			uint32_t sca_start        : 1;
			uint32_t wck_synch        : 1;
			uint32_t private1         : 1;
			uint32_t restart          : 1;
			uint32_t private2         : 24;
		};
	};
};

struct TcmConfigReg
{
	static const int ID = 22;
	TcmConfigReg() {}
	TcmConfigReg(uint32_t r) : value(r) {}

	union
	{
		uint32_t value;
		struct
		{
			uint32_t trig_rate        : 7;
			uint32_t trig_rate_range  : 2;
			uint32_t event_cnt_limit  : 3;
			uint32_t auto_trig_enable : 1;
			uint32_t nim_trig_enable  : 1;
			uint32_t lvds_trig_enable : 1;
			uint32_t ttl_trig_enable  : 1;
			uint32_t sel_edge         : 1;
			uint32_t inv_tcm_sig      : 1;
			uint32_t do_end_of_busy   : 1;
			uint32_t private1         : 1;
			uint32_t event_max_time   : 4;
			uint32_t private2         : 7;
			uint32_t bios_b           : 1;
		};
	};
};

} /* namespace fem */
} /* namespace minos */

#endif /* minos_fem_TcmController_h_INCLUDED */
