/**
 * @file TriggerRate.h
 * @date 8 août 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef minos_fem_TriggerRate_h_INCLUDED
#define minos_fem_TriggerRate_h_INCLUDED

namespace minos
{
namespace fem
{

class TriggerRate
{
public:
	static TriggerRate createFromFreq(double freq);

	TriggerRate();
	TriggerRate(double freq);
	TriggerRate(int range, int rate);

	int range() const { return range_; }
	int rate() const { return rate_; }

	double computeFreq() const;

	static int computeTriggerRange(double freq);
	static int computeTriggerRate(double freq, int range);

private:
	static double computeRangeBaseFreq(int range);

	int range_;
	int rate_;
};

} /* namespace fem */
} /* namespace minos */

#endif /* minos_fem_TriggerRate_h_INCLUDED */
