/**
 * @file MinosRunControl.cpp
 * @date 4 févr. 2014
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the DaqMinos software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "config.h"
#include "MinosRunControl.h"
#include <minos/gui/MinosRunControl.ui.h>
#include <utl/Factory.ipp>
#include <utl/Singleton.ipp>
#include <utl/Construct.hpp>
#include <QCloseEvent>

namespace minos
{
namespace gui
{

MinosRunControl::MinosRunControl(const Params& params, mdcs::acq::ApplicationLoader& loader)
	: mdcs::gui::MasterGuiApplication(params, loader, descriptor()),
	  Utils::Ui::DesignedWidget<Ui_MinosRunControl, QMainWindow>(0)
{
    ui_->runInfoWidget->setMonitor(monitor());
    ui_->pedestalsWidget->setMonitor(monitor());
    ui_->widgetStateMachine->setModel(stateMachineModel_.get());
    ui_->configWindow->setWindowFlags(Qt::Widget);
    ui_->configWindow->setModel(&configSetModel_);

	setTabPosition(Qt::AllDockWidgetAreas, QTabWidget::North);
	centralWidget()->hide();

	LOG_DEBUG() << "restoring MordicusRunControl shape";
	restoreWindowShape(*this);
}

MinosRunControl::~MinosRunControl()
{
}

void MinosRunControl::closeEvent(QCloseEvent* event)
{
	if(event != 0)
	{
		LOG_DEBUG() << "StateMachine: closeEvent";
		ui_->widgetStateMachine->closeEvent(event);
		if(event->isAccepted())
		{
			LOG_DEBUG() << "Config: closeEvent";
			ui_->configWindow->closeEvent(event);
			if(event->isAccepted())
			{
				LOG_DEBUG() << "Saving MordicusRunControl shape";
				saveWindowShape(*this);
				LOG_DEBUG() << "MordicusRunControl: closeEvent";
				QMainWindow::closeEvent(event);
			}
		}
	}
}

const MinosRunControl::Descriptor& MinosRunControl::descriptor()
{
	static const Descriptor d("cea", "irfu.cea.fr", "Minos", VERSION);
	return d;
}

REGISTER_FACTORY(mdcs::acq::ApplicationFactory, "MinosGUI", utl::Construct<MinosRunControl::Params*>())

} /* namespace gui */
} /* namespace minos */
