/* babirl : skelbabies/skelbabies.c
 * last modified : 10/12/01 12:07:29 
 *
 * Hidetada Baba (RIKEN)
 * baba@ribf.riken.jp
 *
 * Event fragment sender (EFS)
 *
 */

#include "RunControlClient.h"
#include "DataThread.h"

#ifndef DEBUG
#define DEBUG
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "ridf.h"
#include "babies.h"

/* common variables */
int comfd;
struct stefrc efrc;
char *data;
int sock = 0, sn;
int efsock = 0;
int fstart;
unsigned int evtn;

/* static values */
volatile int sflag = STAT_RUN_IDLE;
volatile int rtend = STAT_RUN_IDLE;

struct minos_connection* minos = 0;
struct minos_data_thread* mthread = 0;

/** Make TCP server socket (receiver), automatically bind and listen.
 *  Return socket number.
 *  @param port port number should be binded
 *  @return socket number
 */
int mktcpsock(unsigned short port)
{
	int sock = 0;
	int sockopt = 1;
	struct sockaddr_in saddr;

	memset((char *)&saddr, 0, sizeof(saddr));
	if((sock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
	{
		perror("bi-tcp.mktcpsock: Can't make socket.\n");
		return 0;
	}
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof(sockopt));

	saddr.sin_family = AF_INET;
	saddr.sin_addr.s_addr = INADDR_ANY;
	saddr.sin_port = htons(port);
	if(bind(sock, (struct sockaddr *)&saddr, sizeof(saddr)) < 0)
	{
		perror("bi-tcp.mktcpsock: Can't bind socket.\n");
		return 0;
	}
	if(listen(sock, 100) < 0)
	{
		perror("bi-tcp.mktcpsock: Can't listen socket.");
		return 0;
	}

	return sock;
}

/** Make TCP client socket (sender), automatically connect.
 *  Return socket number.
 *  @param host server hostname
 *  @param port port number should be connected
 *  @return socket number
 */
int mktcpsend(char *host, unsigned short port)
{
	int sock = 0;
	struct hostent *hp;
	struct sockaddr_in saddr;

	if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		perror("bi-tcp.mktcpsend: Can't make socket.\n");
		return 0;
	}

	memset((char *)&saddr, 0, sizeof(saddr));

	if((hp = gethostbyname(host)) == NULL )
	{
		printf("bi-tcp.mktcpsend : No such host (%s)\n", host);
		return 0;
	}

	memcpy(&saddr.sin_addr, hp->h_addr, hp->h_length);
	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(port);

	//DB(printf("host : %s  port : %d\n",host,port);)

	if(connect(sock, (struct sockaddr *)&saddr, sizeof(saddr)) < 0)
	{
		perror("bi-tcp.mktcpsend: Error in tcp connect.\n");
		close(sock);
		return 0;
	}

	return sock;
}

void quit(void)
{
	if(minos != 0)
	{
		minos_destroy(&minos);
	}
	if(mthread != 0)
	{
		minos_data_thread_stop(&mthread);
	}
	DB(printf("closing sock\n"));
	if(sock)
		close(sock);
	DB(printf("closing efsock\n"));
	if(efsock)
		close(efsock);

	if(data)
		free(data);

	DB(printf("exit\n"));
	exit(0);
}

// connect to daq server
int efr_connect(void)
{
	int ret;

	ret = 0;
	if(!efrc.connect)
	{
		/* Make data port */
		if(!(efsock = mktcpsend(efrc.erhost, efrc.erport)))
		{
			DB(printf("babies: Can't connect %s:%d\n", efrc.erhost, efrc.erport));
			return 0;
		}
		send(efsock, (char *)&efrc.efid, sizeof(efrc.efid), 0);
		recv(efsock, (char *)&ret, sizeof(ret), MSG_WAITALL);
		efrc.connect = 1;
		DB(printf("skelbabies: EFR return = %d\n", ret));
	}
	else
	{
		DB(printf("skelbabies: EFR already connected\n"));
		return 0;
	}

	return 1;
}

int efr_disconnect(void)
{
	if(efrc.connect)
	{
		close(efsock);
		efrc.connect = 0;
	}
	else
	{
		DB(printf("skelbabies: EFR not connected\n"));
		return 0;
	}
	return 1;
}

// send dummy data
void senddummydata()
{
	RIDFHD hd;
	RIDFHDEVTTS evthd;
	RIDFHDSEG seghd;
	int evtlen, evts, lots, segs, mp, segid, len;

	mp = sizeof(hd);

	segid = 0x00123535; // zds, f8, minos, minos for segment id

	evtn++;
	DB(printf("evtn=%d\n", evtn));
	evtlen = 100; //100 short word

	evts = evtlen + sizeof(evthd) / sizeof(short) + sizeof(seghd) / sizeof(short);
	lots = evtn;  // timestamp = evtn for here
	evthd = ridf_mkhd_evtts(RIDF_LY1, RIDF_EVENT_TS, evts, efrc.efid, evtn, lots);
	memcpy((char *)(data + mp), (char *)&evthd, sizeof(evthd));
	mp += sizeof(evthd);
	segs = evtlen + sizeof(seghd) / sizeof(short);
	seghd = ridf_mkhd_seg(RIDF_LY2, RIDF_SEGMENT, segs, efrc.efid, segid);
	memcpy((char *)(data + mp), (char *)&seghd, sizeof(seghd));
	mp += sizeof(seghd);
	memset((char *)(data + mp), 0x01, evtlen * 2);
	mp += evtlen;

	hd = ridf_mkhd(RIDF_LY0, RIDF_EF_BLOCK, mp / sizeof(short), efrc.efid);
	memcpy((char *)data, (char *)&hd, sizeof(hd));
	// here, mp is the size of 1 block

	if(!efrc.connect)
	{
		efr_connect();
	}
	if(efrc.connect)
	{
		DB(printf("send data to EFR size=%d\n", mp));
		memcpy((char *)&len, data, sizeof(len));
		len = RIDF_SZ(len) * 2;  //get buff length from data
		send(efsock, (char *)&len, sizeof(len), 0); //send the length of this block
		send(efsock, data, len, 0);  //send the data contents
	}
}

// command from DAQ server
int commain(void)
{
	struct sockaddr_in caddr;
	char buff[BABIRL_COM_SIZE], tdir[80] = {0}, f[80] = {0};
	int sock, clen, len;
	int com, ret, arg;

	clen = sizeof(caddr);
	if((sock = accept(comfd, (struct sockaddr *)&caddr, (socklen_t *)&clen)) < 0)
	{
		perror("Error in accept commain\n");
		return 0;
	}

	memset(buff, 0, sizeof(buff));
	recv(sock, (char *)&len, sizeof(len), MSG_WAITALL);
	recv(sock, buff, len, MSG_WAITALL);

	/* babies command */
	memcpy((char *)&com, buff, sizeof(com));

	switch(com)
	{
	case ES_SET_CONFIG:
		DB(printf("babies: ES_SET_CONFIG\n"));
		memset(tdir, 0, sizeof(tdir));
		strcpy(tdir, efrc.mtdir);

		memcpy((char *)&efrc, buff + sizeof(com), sizeof(efrc));
		memset(buff, 0, sizeof(buff));
		ret = 1;
		memcpy(buff, (char *)&ret, sizeof(ret));
		len = sizeof(ret);
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, buff, len, 0);
		efr_disconnect();
		break;
	case ES_GET_CONFIG:
		DB(printf("babies: ES_GET_CONFIG\n"));
		memset(buff, 0, sizeof(buff));
		memcpy(buff, (char *)&efrc, sizeof(efrc));
		len = sizeof(efrc);
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, buff, len, 0);
		break;
	case ES_RELOAD_DRV:
		// load driver
		break;
	case ES_RUN_START:
	{
		DB(printf("babies: ES_RUN_START\n"));
		memcpy((char *)&arg, buff + sizeof(com), sizeof(arg));
		if(arg == ES_EF_OFF && !fstart)
		{
			DB(printf("skelbabies: Run started, but not joining event build\n"));
			break;
		}

		minos_error minos_status = minos_start(minos);
		// event number = 0
		evtn = 0;
		// when start ready, return start ack
		if(minos_status == minos_ok)
		{
			ret = 1;
		}
		else
		{
			ret = -1;
		}
		len = sizeof(ret);
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, (char *)&ret, len, 0);
		sflag = STAT_RUN_START;

		break;
	}
	case ES_RUN_NSSTA:
	{
		// run start with no save mode
		DB(printf("babies: ES_NSRUN_START\n"));
		memcpy((char *)&arg, buff + sizeof(com), sizeof(arg));
		if(arg == ES_EF_OFF && !fstart)
		{
			DB(printf("babies: Run started, but not joining event build\n"));
			break;
		}

		minos_error minos_status = minos_start(minos);

		// event number = 0
		evtn = 0;
		// when start ready, return start ack
		if(minos_status == minos_ok)
		{
			ret = 1;
		}
		else
		{
			ret = -1;
		}
		len = sizeof(ret);
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, (char *)&ret, len, 0);
		sflag = STAT_RUN_NSSTA;

		break;
	}
	case ES_RUN_STOP:
		DB(printf("babies: ES_RUN_STOP\n"));

		if(sflag != STAT_RUN_IDLE)
		{
			DB(printf("skelbabies: evtn=%d\n", evtn));
			if(!efrc.connect)
			{
				efr_connect();
			}
			minos_stop(minos);
			if(efrc.connect)
			{
				DB(printf("End of run\n"));
				// return stop ack (-1) to the server
				len = -1;
				send(efsock, (char *)&len, sizeof(len), 0);

//				efr_disconnect();
			}
			sflag = STAT_RUN_IDLE;

			// return command ack
			ret = 1;
			len = sizeof(ret);
			send(sock, (char *)&len, sizeof(len), 0);
			send(sock, (char *)&ret, len, 0);
		}
		break;
	case ES_GET_EVTN:
		// return the event number
		ret = evtn;
		len = sizeof(ret);
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, (char *)&ret, len, 0);
		break;
	case WHOAREYOU:
		DB(printf("skelbabies : WHOAREYOU\n"));
		memset(f, 0, sizeof(f));
		len = sprintf(f, "skelbabies");
		send(sock, (char *)&len, sizeof(len), 0);
		send(sock, f, len, 0);
		break;
	case ES_QUIT:
		DB(printf("skelbabies: ES_QUIT\n"));
		if(!sflag)
			quit();
		break;
	case ES_CON_EFR:
		DB(printf("skelbabies: ES_CON_EFR\n"));
		minos_connect(minos);
		minos_data_thread_start(&mthread);
		if(!efr_connect())
		{
			DB(printf("skelbabies: EFR connection faild\n"));
		}
		break;
	case ES_DIS_EFR:
		DB(printf("skelbabies: ES_DIS_EFR\n"));
		minos_data_thread_stop(&mthread);
		minos_disconnect(minos);
		if(!efr_disconnect())
		{
			DB(printf("skelbabies: EFR disconnection faild\n"));
		}
		break;
	}

	if(sock)
		close(sock);
	return 1;
}

int main(int argc, char *argv[])
{

	minos = minos_create(&argc, argv);
	if(minos == 0)
	{
		printf("unable to initialize minos_connection, aborting");
		return -1;
	}

	fd_set fdset;

	if(argc != 3)
	{
		printf("skelbabies EFID HOSTNAME\n");
		printf("\n\n for example \n skelbabies 20 localhost\n");
		exit(0);
	}

	fstart = 0;
	evtn = 0;

	efrc.efid = strtol(argv[1], NULL, 0);
	if(efrc.efid < 0 || efrc.efid > MAXEF)
	{
		printf("skelbabies: Error 0 < EFN < %d\n", MAXEF);
		exit(0);
	}
	printf("skelbabies: EFN = %d\n", efrc.efid);

	efrc.runnumber = -1;
	efrc.erport = ERRCVPORT;
	efrc.comport = ESCOMPORT + efrc.efid;
	memset(efrc.erhost, 0, sizeof(efrc.erhost));
	strcpy(efrc.erhost, argv[2]);
	efrc.hd1 = FALSE;
	strcpy(efrc.hd1dir, "./\0");
	efrc.hd2 = FALSE;
	strcpy(efrc.hd2dir, "./\0");
	efrc.mt = FALSE;
	strcpy(efrc.mtdir, "./\0");
	strcpy(efrc.runname, "\0");
	efrc.connect = 0;

	data = malloc((size_t)EB_EFBLOCK_BUFFSIZE);
	memset(data, 0, EB_EFBLOCK_BUFFSIZE);

	/* Signal SIGINT -> quit() */
	signal(SIGINT, (void *)quit);

	sflag = STAT_RUN_IDLE;

	/* Make command port */
	if(!(comfd = mktcpsock(ESCOMPORT + efrc.efid)))
		quit();
	DB(printf("skelbabies: comfd number = %d\n", comfd));

	while(1)
	{
		DB(printf("skelbabies: Main\n"));

		sn = comfd;
		FD_ZERO(&fdset);
		FD_SET(comfd, &fdset);
		if(select(sn + 1, &fdset, NULL, NULL, NULL ) != 0)
		{
			DB(printf("came****\n"));
			if(FD_ISSET(comfd, &fdset))
			{
				DB(printf("select : commain\n"));
				commain();
			}
			else
			{
				usleep(1);
			}
		}
	}

	return 0;
}

