/**
 * @file: MemBusDemo.hpp
 * @created: Feb 18, 2013
 * @author: anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licensing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#ifndef demo_mdcs_hw_MemBusDemo_hpp_INCLUDED
#define demo_mdcs_hw_MemBusDemo_hpp_INCLUDED

#include "mdcs/hw/policy/MemBus/Policy.hpp"
#include "mdcs/hw/DeviceT.hpp"
#include <utl/BitSet.hpp>
#include <boost/mpl/if.hpp>

namespace demo
{
namespace mdcs
{
namespace hw
{

class MemPool;

typedef ::mdcs::hw::policy::MemBus::BasePolicy::AddrType    AddrType;
typedef ::mdcs::hw::policy::MemBus::BasePolicy::Offset      Offset;
typedef ::mdcs::hw::policy::MemBus::BasePolicy::InDeviceRef InDeviceRef;
typedef ::mdcs::hw::BitField BitField;

template< size_t NBITS, ::utl::Signedness SGNSS >
struct RegConst
{
	static size_t get_NBITS() { return NBITS; }
	static bool is_SIGNED() { return SGNSS == ::utl::SIGNED; }
};

template< size_t NBITS, ::utl::Signedness SGNSS >
struct RegWord : public RegConst<NBITS,SGNSS>
{
	typedef ::utl::BitSet<NBITS,SGNSS> DataType;
	static size_t get_NBYTES() { return ::utl::BitSet<NBITS, SGNSS>::minByteCount(); }
};

template< ::utl::Signedness SGNSS >
struct RegWord<0u,SGNSS> : public RegConst<0u,SGNSS>
{
	typedef ::utl::RtBitSet<SGNSS> DataType;
	static size_t get_NBYTES() { return ::utl::BitSet<0u, SGNSS>::minByteCount(); }
};

template< ::utl::Signedness SGNSS >
struct RegWord<8u,SGNSS> : public RegConst<8u,SGNSS>
{
	typedef typename boost::mpl::if_c< SGNSS == ::utl::SIGNED, int8_t, uint8_t >::type DataType;
	static size_t get_NBYTES() { return sizeof(DataType); }
};

template< ::utl::Signedness SGNSS >
struct RegWord<16u,SGNSS> : public RegConst<16u,SGNSS>
{
	typedef typename boost::mpl::if_c< SGNSS == ::utl::SIGNED, int16_t, uint16_t >::type DataType;
	static size_t get_NBYTES() { return sizeof(DataType); }
};

template< ::utl::Signedness SGNSS >
struct RegWord<32u,SGNSS> : public RegConst<32u,SGNSS>
{
	typedef typename boost::mpl::if_c< SGNSS == ::utl::SIGNED, int32_t, uint32_t >::type DataType;
	static size_t get_NBYTES() { return sizeof(DataType); }
};

template< ::utl::Signedness SGNSS >
struct RegWord<64u,SGNSS> : public RegConst<64u,SGNSS>
{
	typedef typename boost::mpl::if_c< SGNSS == ::utl::SIGNED, int64_t, uint64_t >::type DataType;
	static size_t get_NBYTES() { return sizeof(DataType); }
};

template<size_t NBITS, ::utl::Signedness SGNSS, typename RegKey, typename BitFieldKey>
struct MemBusDemo : public RegWord<NBITS, SGNSS>
{
	typedef RegWord<NBITS, SGNSS> Base;
	typedef ::mdcs::hw::policy::MemBus::Policy< typename Base::DataType > Policy;
	typedef ::mdcs::hw::RegisterT < BitFieldKey, Policy   > Register;
	typedef ::mdcs::hw::DeviceT   < RegKey, Register > Device;
	typedef ::mdcs::hw::BitField::Pool<BitFieldKey> BitFieldPool;

	static const RegKey REG_CONTROL;
	static const RegKey REG_CONFIGS;

	static const BitFieldKey BF_LSB;
	static const BitFieldKey BF_MSB;

	static size_t get_NBITS() { return RegConst<NBITS,SGNSS>::get_NBITS(); }
	static bool is_SIGNED() { return RegConst<NBITS,SGNSS>::is_SIGNED(); }
	static size_t get_NBYTES() { return RegWord<NBITS,SGNSS>::get_NBYTES(); }

	static void run(MemPool& memPool);
};

} // namespace hw
} // namespace mdcs
} // namespace demo

#endif /* demo_mdcs_hw_MemBusDemo_hpp_INCLUDED */
