/**
 * @file MemBusDemo.ipp
 * @created Oct 30, 2012
 * @author anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#ifndef demo_mdcs_hw_MemBusDemo_ipp_INCLUDED
#define demo_mdcs_hw_MemBusDemo_ipp_INCLUDED

#include "MemBusDemo.hpp"
#include "MemPool.h"
#include "mdcs/hw/policy/MemBus/Policy.ipp"
#include "mdcs/hw/DeviceT.ipp"
#include "mdcs/utl/TypeInfo.hpp"
#include <utl/Logging.h>
#include <iomanip>

namespace demo
{
namespace mdcs
{
namespace hw
{

template<size_t NBITS, ::utl::Signedness SGNSS, typename RegKey, typename BitFieldKey>
void MemBusDemo<NBITS, SGNSS, RegKey, BitFieldKey>::run(MemPool& memPool)
{
	typedef ::mdcs::utl::TypeInfo<typename MemBusDemo::DataType> TypeInfo;

	LOG_INFO() << "Starting " << Policy::name() << " Device Demo";
	LOG_INFO() << "Allocated memory @ " << (memPool.getBase<void>());
	memPool.dump<uint8_t>(16);

	LOG_INFO() << "Creating device & registers based on data type: " << TypeInfo::name();

	Device device(TypeInfo::name() + "-0");

	device.newReg(REG_CONTROL, InDeviceRef(memPool.getBase<void>(), (Offset) 0)); // Creating register control @ offset 0
	device.newReg(REG_CONFIGS, InDeviceRef(memPool.getBase<void>(), (Offset) get_NBYTES())); // Creating register configs @ offset 4

	LOG_INFO() << "Setting registers to 0";
	device[REG_CONTROL] = 0u;
	device[REG_CONFIGS] = 0u;
	memPool.dump<uint8_t>(16);

	LOG_INFO() << "Setting registers to 0xCAFEFADE and 0xDEADFACE";
	device[REG_CONTROL] = 0xCAFEFADE;
	device[REG_CONFIGS] = 0xDEADFACE;
	memPool.dump<uint8_t>(16);

	LOG_INFO() << "Creating bit fields for register: " << REG_CONTROL;

	device[REG_CONTROL].bitFieldPool()
			.mapBitField(BF_LSB, BitField( 0, get_NBITS()/2))
			.mapBitField(BF_MSB, BitField( get_NBITS()/2, get_NBITS()-get_NBITS()/2));

	LOG_INFO() << "Creating bit fields for register: " << REG_CONFIGS;
	device[REG_CONFIGS].bitFieldPool()
			.mapBitField(BF_LSB, BitField( 0, get_NBITS()/2))
			.mapBitField(BF_MSB, BitField( get_NBITS()/2, get_NBITS()-get_NBITS()/2));

	typename BitFieldPool::KeySet bfKeySet = device[REG_CONTROL].bitFieldPool().keySet();
	for (typename BitFieldPool::KeySet::iterator it = bfKeySet.begin(); it != bfKeySet.end(); ++it)
	{
		LOG_INFO() << "BitField key: " << *it;
	}

	LOG_INFO() << "Writing bit fields in: " << REG_CONTROL;
	device[REG_CONTROL]
		.poke(BF_LSB  , 0xa)
		.poke(BF_MSB  , 0xb);
	memPool.dump<uint8_t>(16);

	LOG_INFO() << "Writing bit fields in: " << REG_CONFIGS;
	device[REG_CONFIGS]
		.poke(BF_LSB  , 0xfb)
		.poke(BF_MSB  , 0xfc);
	memPool.dump<uint8_t>(16);

	LOG_INFO() << "End of " << Policy::name() << " Device Demo";
}

} // namespace hw
} // namespace mdcs
} // namespace demo

#endif /* demo_mdcs_hw_MemBusDemo_ipp_INCLUDED */
