/**
 * @file MemBusMain.cpp
 * @created Dec 14, 2012
 * @author anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#include "PrettyStdoutLogging.h"
#include "MemBusDemo.ipp"

using namespace std;

static ::demo::mdcs::hw::MemPool memPool(128u, (uint8_t) 1, (uint8_t) 1);

static const size_t PARTIAL_NBITS = 20;
static const ::utl::Signedness PARTIAL_SIGNEDNESS = ::utl::SIGNED;
static const size_t ROUND_NBITS = 32;
static const ::utl::Signedness ROUND_SIGNEDNESS = ::utl::UNSIGNED;

namespace demo
{
namespace mdcs
{
namespace hw
{

template<> const int MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, int, int>::REG_CONTROL = 0;
template<> const int MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, int, int>::REG_CONFIGS = 1;
template<> const int MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, int, int>::BF_LSB      = 10;
template<> const int MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, int, int>::BF_MSB      = 11;

template<> const std::string MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, std::string, std::string>::REG_CONTROL = "control";
template<> const std::string MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, std::string, std::string>::REG_CONFIGS = "configs";
template<> const std::string MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, std::string, std::string>::BF_LSB = "BF_LSB";
template<> const std::string MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, std::string, std::string>::BF_MSB = "BF_MSB";

template<> const int MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, int, int>::REG_CONTROL = 0;
template<> const int MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, int, int>::REG_CONFIGS = 1;
template<> const int MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, int, int>::BF_LSB      = 10;
template<> const int MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, int, int>::BF_MSB      = 11;

template<> const std::string MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, std::string, std::string>::REG_CONTROL = "control";
template<> const std::string MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, std::string, std::string>::REG_CONFIGS = "configs";
template<> const std::string MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, std::string, std::string>::BF_LSB = "BF_LSB";
template<> const std::string MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, std::string, std::string>::BF_MSB = "BF_MSB";

} // namespace hw
} // namespace mdcs
} // namespace demo

int main()
{
	::utl::BackendLogger::setBackend(::utl::LoggingBackendPtr(new ::demo::mdcs::hw::PrettyStdoutLogging));
	try
	{
		LOG_INFO() << string(120, '=');
		memPool.init(0xFF,0);
		demo::mdcs::hw::MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, int, int>::run(memPool);

		LOG_INFO() << string(120, '-');
		memPool.init(0xFF,0);
		demo::mdcs::hw::MemBusDemo<PARTIAL_NBITS, PARTIAL_SIGNEDNESS, string, string>::run(memPool);

		LOG_INFO() << string(120, '-');
		memPool.init(0xFF,0);
		demo::mdcs::hw::MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, int, int>::run(memPool);

		LOG_INFO() << string(120, '-');
		memPool.init(0xFF,0);
		demo::mdcs::hw::MemBusDemo<ROUND_NBITS, ROUND_SIGNEDNESS, string, string>::run(memPool);
	}
	catch(const exception& e)
	{
		LOG_ERROR() << e.what();
	}
	catch(const string& e)
	{
		LOG_ERROR() << e;
	}
	catch(const char* e)
	{
		LOG_ERROR() << e;
	}
	catch(...)
	{
		LOG_ERROR() << "Unknown exception";
	}

	LOG_INFO() << string(120, '=');
	LOG_INFO() << "Bye!";
	return 0;
}
