/**
 * @file MemPool.h
 * @created Dec 14, 2012
 * @author anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#ifndef demo_mdcs_hw_MemPool_h_INCLUDED
#define demo_mdcs_hw_MemPool_h_INCLUDED

#include "utl/WordIterator.hpp"
#include <boost/cstdint.hpp>
#include <vector>
#include <iostream>
#include <iomanip>

namespace demo
{
namespace mdcs
{
namespace hw
{

class MemPool
{
public:
	MemPool(size_t nBytes, uint8_t initVal = 0u, uint8_t stepVal = 0u);
	virtual ~MemPool();

	void init(uint8_t val, uint8_t stepVal = 0);

	template < typename W >
	W* getBase() { return bytePool.size() == 0? 0 : reinterpret_cast<W*>(&bytePool[0]); }

	template < typename W >
	W& getWord(size_t i) { return getBase<W>()[i]; }

	template < typename W >
	void dump(size_t nWords)
	{
		std::cout << ">> MemPool hex dump:";
		::utl::WordIterator<W> witer = ::utl::WordIterator<W>::getBegin(getBase<W>(), nWords);
		::utl::WordIterator<W> wend  = ::utl::WordIterator<W>::getEnd(getBase<W>(), nWords);

		while (witer != wend)
		{
			std::cout << " " << std::hex << std::setfill('0') << std::setw(2*sizeof(W)) << ((unsigned)*witer);
			++witer;
		}
		std::cout << std::dec << std::endl;
	}


private:
	std::vector<uint8_t> bytePool;
};

} // namespace hw
} // namespace mdcs
} // namespace demo

#endif /* demo_mdcs_hw_MemPool_h_INCLUDED */
