/**
 * @file AcqCCfg.cpp
 * @date 22 juil. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "AcqCCfg.h"
#include "RunInfo.h"
#include "ServerApplication.h"
#include "ApplicationLoader.h"
#include <mdcs/cfg/CCfgConfig.h>
#include <mdcs/cfg/IdentityProvider.h>
#include <CCfg/CConfig.h>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/lexical_cast.hpp>
#include <utl/Logging.h>

using namespace boost::posix_time;

namespace mdcs
{
namespace acq
{
using namespace CCfg;


template<>
void RunInfo::decode(RunInfo& obj, const cfg::CCfgConfig& config)
{
	CConfig cfg = config.root();
	obj.runSite_      = cfg("site").getStringValue();
	obj.runId_        = cfg("runId");
	obj.runTimestamp_ = boost::lexical_cast<RunTimestamp>(cfg("timestamp"));
	obj.fileStem_     = cfg("fileStem").getStringValue();

	LOG_INFO() << "RunInfo decoded: site=" << obj.runSite_ << " id=" << obj.runId_ << " stem=" << obj.fileStem_;
}

template<>
void RunInfo::encode(cfg::CCfgConfig& config, const RunInfo& obj)
{
	CConfig cfg = config.root();
	cfg.setName("RunInfo");
	cfg("site")      = obj.runSite_;
	cfg("runId")     = obj.runId_;
	cfg("timestamp") = boost::lexical_cast<std::string>(obj.runTimestamp_);
	cfg("fileStem")  = obj.fileStem_;
}

template<>
void ApplicationLoader::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig cfg = subconfig;
	p.applicationName = cfg("applicationName").getStringValue();
	p.configFormat    = cfg("configFormat").getStringValue();
	p.logLevel        = cfg("Logging")("maxLogLevel");

	LOG_DEBUG() << "Looking for plugins";

	for(CConfig::Iterator iterator = cfg("Plugins").iterate(); iterator.hasNext();)
	{
		CConfig plugin = iterator.next();
		LOG_DEBUG() << "discovered plugin: " << plugin.getName() << '=' << plugin.getStringValue();
		p.plugins.push_back(plugin.getStringValue());
	}
}

template<>
void ApplicationLoader::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

template<>
void ServerApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig cfg = subconfig;
	p.primaryAdapter  = cfg("primaryAdapter").getStringValue();
	p.samplingRate_ms = cfg("Monitoring")("samplingRate");
}

CCfgInit::CCfgInit()
{
	cfg::Configurator::instance()
		.add(&RunInfo::decode<cfg::CCfgConfig>)
		.add(&RunInfo::encode<cfg::CCfgConfig>)
		.add(&ApplicationLoader::Params::init<cfg::CCfgConfig>)
		.add(&ApplicationLoader::Params::init<CCfg::CConfig>)
		.add(&ServerApplication::Params::init<CCfg::CConfig>)
		;
}

} /* namespace acq */
} /* namespace mdcs */
