/**
 * @file Application.h
 * @date 23 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_Application_h_INCLUDED
#define mdcs_acq_Application_h_INCLUDED

#include <utl/Factory.hpp>
#include <utl/Singleton.hpp>
#include ICE_HEADER(Communicator.h)
#include <boost/scoped_ptr.hpp>
#include <boost/utility.hpp>
#include <string>
#include <memory>

namespace mdcs
{
namespace acq
{
// forward declarations
class ApplicationLoader;
class Application;

typedef std::auto_ptr<Application> ApplicationPtr;

class ApplicationParams
{
public:
	virtual ~ApplicationParams();
	virtual ApplicationPtr createApplication(ApplicationLoader& loader) const = 0;
	template<class C> static void init(ApplicationParams& /*obj*/, const C& /*cfg*/) {}
};

class Application : boost::noncopyable
{
public:
	typedef ApplicationParams Params;

	Application(const Params& params, ApplicationLoader& loader);
	virtual ~Application();

	virtual void mainLoop() = 0;

protected:
	ApplicationLoader& loader_;
};

template<class DerivedApp, class BaseApp = Application>
class ApplicationParamsHelper : public BaseApp::Params
{
public:
	virtual ~ApplicationParamsHelper() {}
	virtual ApplicationPtr createApplication(ApplicationLoader& loader) const
	{
		return ApplicationPtr(new DerivedApp(static_cast<const typename DerivedApp::Params&>(*this), loader));
	}
};

typedef ::utl::Singleton< ::utl::FactoryMap<std::string, ApplicationParams*()> > ApplicationFactory;

} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_Application_h_INCLUDED */
