/**
 * @file PolicyStates.hpp
 * @date 25 juil. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_PolicyStates_hpp_INCLUDED
#define mdcs_acq_PolicyStates_hpp_INCLUDED

#include <mdcs/acq/ServerApplication.h>
#include <mdcs/utl/PolicyTuple.hpp>
#include <mdcs/utl/Tuples.hpp>
#include <boost/static_assert.hpp>
#include <boost/type_traits/is_base_of.hpp>
#include <boost/type_traits/is_same.hpp>
#include <typeinfo>
#include <string>
#include <iostream>

namespace mpl = boost::mpl;


namespace mdcs
{
namespace acq
{

template<class PolicyVector>
class ConfiguredPolicies : public ConfiguredState
{
public:
	typedef typename utl::PolicyTuple<PolicyVector, utl::ParamsHolder>::type PolicyParamsTuple;
	typedef typename utl::PolicyTuple<PolicyVector, utl::SetupHolder >::type PolicySetupTuple;

	ConfiguredPolicies(const StateData& base) : ConfiguredState(base) {}
	virtual ~ConfiguredPolicies() {}

	template<class C> static void init(ConfiguredPolicies& obj, const C& config)
	{
		utl::tuple_unary_apply(obj.policiesParams_, utl::PolicyTupleFunc::init(config));
	}

	PolicySetupTuple& policiesSetup() { return policiesSetup_; }

protected:
	virtual void apply()
	{
		utl::tuple_binary_apply(policiesSetup_, policiesParams_, utl::PolicyTupleFunc::create(*this));
	}

private:
	PolicyParamsTuple policiesParams_;
	PolicySetupTuple policiesSetup_;
};

template<class PolicyVector>
class StartedPolicies : public StartedState
{
public:
	typedef typename utl::PolicyTuple<PolicyVector, utl::RunImplHolder>::type PolicyRunImplTuple;

	StartedPolicies(const StateData& base) : StartedState(base) {}
	virtual ~StartedPolicies() {}

	ConfiguredPolicies<PolicyVector>& configuredCtx()
	{
		return dynamic_cast<ConfiguredPolicies<PolicyVector>&>(*context());
	}

protected:
	virtual void doStart()
	{
		utl::tuple_binary_apply(policies_, configuredCtx().policiesSetup(), utl::PolicyTupleFunc::create(*this, runInfo()));
	}

	PolicyRunImplTuple policies_;
};

} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_PolicyStates_hpp_INCLUDED */
