/**
 * @file RunInfo.h
 * @date 22 juil. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_RunInfo_h_INCLUDED
#define mdcs_acq_RunInfo_h_INCLUDED

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/cstdint.hpp>
#include <string>
#include <memory>
#include <ctime>

namespace mdcs
{
namespace acq
{

typedef uint32_t RunId;
typedef boost::posix_time::ptime RunTimestamp;

class RunInfo
{
public:
	RunInfo();
	RunInfo(std::string site, RunId id, RunTimestamp ts, const std::string& fileStem);
	virtual ~RunInfo();

	const std::string& runSite() const { return runSite_; }
	RunId runId() const { return runId_; }
	RunTimestamp runTimestamp() const { return runTimestamp_; }
	const std::string& fileStem() const { return fileStem_; }

	template<class C> static void decode(RunInfo& obj, const C& cfg);
	template<class C> static void encode(C& cfg, const RunInfo& obj);

private:
	std::string runSite_;
	RunId runId_;
	RunTimestamp runTimestamp_;
	std::string fileStem_;
};

typedef std::auto_ptr<RunInfo> RunInfoPtr;

} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_RunInfo_h_INCLUDED */
