/**
 * @file ServerApplication.h
 * @date 23 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_ServerApplication_h_INCLUDED
#define mdcs_acq_ServerApplication_h_INCLUDED

#include "Application.h"
#include "StateMachineServer.h"
#include "RunInfo.h"
#include <dhsm/StateData.h>

namespace mdcs
{
namespace acq
{
// forward declarations
namespace mon { class MetricsCollector; }
class DistributedEngineCreator;

class ServerApplication : public Application
{
public:
	class Params;

	ServerApplication(const Params& params,
		ApplicationLoader& app,
		const DistributedEngineCreator& creator,
		const std::string& identity,
		const std::string& stateMachineName);
	virtual ~ServerApplication();

	const StateMachineServer& server() const { return server_; }
	const Ice::ObjectAdapterPtr& adapter() const { return adapter_; }
	mon::MetricsCollector& collector() { return *collector_; }

	class SmContext;

	void createTopContext(const StateMachineServer& server);

	virtual void mainLoop();

private:
	static Ice::ObjectAdapterPtr createAdapter(const ApplicationLoader& loader, const Params& params);
	void configureLogger();

	Ice::ObjectAdapterPtr adapter_;
	StateMachineServer server_;
	boost::scoped_ptr<mon::MetricsCollector> collector_;
};

class ServerApplication::Params : public ApplicationParams
{
public:
	template<class C> static void init(Params& p, const C& config);

	virtual ~Params();

	std::string primaryAdapter;
	int samplingRate_ms;
};

class ServerApplication::SmContext : public dhsm::StateData
{
public:
	SmContext(ServerApplication& application) : application_(application) {}

	ServerApplication& application() { return application_; }
	template<class A> A& application() { return dynamic_cast<A&>(application_); }

private:
	ServerApplication& application_;
};

class ApplicationState : public dhsm::StateData
{
public:
	ApplicationState(const StateData& base);
	virtual ~ApplicationState();

	ServerApplication& application();
	template<class A> A& application() { return dynamic_cast<A&>(application()); }
};

class ConfiguredState : public ApplicationState
{
public:
	ConfiguredState(const StateData& base);
	virtual ~ConfiguredState();
	void configure(const std::string& config);

protected:
	virtual void apply() = 0;
};

class StartedState : public ApplicationState
{
public:
	StartedState(const StateData& base);
	virtual ~StartedState();
	void start(const std::string& runCfg);
	const RunInfo& runInfo() const { return runInfo_; }

protected:
	virtual void doStart() = 0;

private:
	RunInfo runInfo_;
};

} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_ServerApplication_h_INCLUDED */
