/* =============================================================================
 * StateMachineServer.cpp
 * -----------------------------------------------------------------------------
 * class mdcs::acq::StateMachineServer
 * Created on: 28 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include "StateMachineServer.h"
#include <mdcs/acq/StateMachineFactory.h>
#include <dhsm/EngineServant.h>
#include ICE_HEADER(Communicator.h)

namespace mdcs
{
namespace acq
{

using namespace dhsm;

StateMachineServer::StateMachineServer(
		const DistributedEngineCreator& creator,
		const std::string& identity,
		const std::string& stateMachineName,
		Ice::ObjectAdapterPtr adapter)
	: adapter_(adapter), identity_(identity)
{
	LOG_INFO() << "Creating state-machine: " << stateMachineName;
	StateMachinePtr stateMachine = StateMachineFactory::instance()[stateMachineName].create();
	stateMachine->label(identity);

	EnginePtr engine = creator.createSmEngine(*this);
	engine->setStateMachine(stateMachine);
	engine_ = engine.get();

	std::auto_ptr<dhsm::EngineServant> servant = creator.createSmServant(*this);
	servant->setEngine(engine);
	Ice::ObjectPtr object = servant.release();

	LOG_INFO() << "Adding state-machine servant: " << identity;
	adapter->add(object, adapter->getCommunicator()->stringToIdentity(identity));
}

StateMachineServer::~StateMachineServer()
{
	Ice::Identity id;
	id.name  = identity_;
	adapter_->remove(id);
}

} /* namespace acq */
} /* namespace mdcs */
