/* =============================================================================
 * AcqBackCCfg.cpp
 * -----------------------------------------------------------------------------
 * Created on: 28 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include "AcqBackCCfg.h"
#include <mdcs/cfg/CCfgUtils.ipp>
#include "DataProcessorApplication.h"
#include "policy/DataPipeline.h"
#include "policy/IceDataReceiver.h"
#include "policy/FileStorage.h"
#include <mdcs/cfg/CCfgConfig.h>
#include <mdcs/cfg/IdentityProvider.h>
#include <mdcs/utl/PolicyTuple.hpp>
#include <mdcs/utl/Tuples.hpp>
#include <CCfg/Object.h>
#include <utl/Logging.h>

namespace mdcs
{
namespace acq
{
namespace back
{

template<>
void DataProcessorApplication::Configured::init(Configured& obj, const cfg::CCfgConfig& config)
{
	CCfg::CConfig root = config.root();
	CCfg::CConfig self = root("DataProcessor", cfg::IdentityProvider::instance().id().toString());
	obj.daqGroup = self("daqGroup").getStringValue();

	ConfiguredPolicies<Policies>::init(obj, self);
	
	LOG_DEBUG() << "configured DataProcessorApplication";
}

template<>
void DataProcessorApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	ServerApplication::Params::init(p, subconfig);
}

template<>
void DataProcessorApplication::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

namespace policy
{

template<>
void DataPipeline::Params::init(Params& obj, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	obj.dataGroup = config("dataGroup").getStringValue();

	utl::CompositeParamData<Policy, NestedPolicies>::init(obj, config);

	LOG_DEBUG() << "configured DataPipeline";
}

template<>
void IceDataReceiver::Params::init(Params& r, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	r.collocated = config("collocated");
	LOG_DEBUG() << "configured IceDataReceiver";
}

template<>
void FileStorage::Params::init(Params& s, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	s.dataDir       = config("datadir").getStringValue();
	s.storeData     = config("isStoring");
	s.limitFileSize = config("limitFileSize");
	s.maxFileSize   = config("maxFileSize").getHexValue();
	s.seqDigitCount = config("seqDigitCount");
	s.filePrefix    = config("filePrefix").getStringValue();
	s.fileSuffix    = config("fileSuffix").getStringValue();
}

} /* namespace policy */

CCfgInit::CCfgInit()
{
	cfg::Configurator::instance()
		.add(&DataProcessorApplication::Configured::init<cfg::CCfgConfig>)
		.add(&DataProcessorApplication::Params::init<cfg::CCfgConfig>)
		.add(&DataProcessorApplication::Params::init<CCfg::CConfig>)
		.add(    &policy::DataPipeline::Params::init<CCfg::CConfig>)
		.add( &policy::IceDataReceiver::Params::init<CCfg::CConfig>)
		.add(     &policy::FileStorage::Params::init<CCfg::CConfig>)
		;
}

} /* namespace back */
} /* namespace acq */

namespace utl
{
template<>
CCfg::CConfig getPolicyConfig(const PolicyParams<acq::back::DataProcessor>&, CCfg::CConfig inConfig, const std::string& /*metaPolicyName*/, std::string& policyClass)
{
	policyClass = inConfig("policy").getStringValue();
	return inConfig(policyClass);
}

CCfg::CConfig::Iterator policyIterate(CCfg::CConfig inConfig, const std::string& metaPolicyName)
{
	return inConfig.iterate(metaPolicyName);
}

CCfg::CConfig policyNext(CCfg::CConfig::Iterator& iterator, std::string& policyClass)
{
	CCfg::CConfig& cfg = iterator.get();
	policyClass = cfg("policy").getStringValue();
	return cfg(policyClass);
}

bool policyHasNext(CCfg::CConfig::Iterator& iterator)
{
	while(iterator.hasNext())
	{
		CCfg::CConfig& cfg = iterator.next();
		if(cfg.getIndex() != "*" and cfg("isActive").getBoolValue() == true)
		{
			return true;
		}
	}
	return false;
}

} /* namespace utl */
} /* namespace mdcs */
