/* =============================================================================
 * DataProcessorApplication.cpp
 * -----------------------------------------------------------------------------
 * class mdcs::acq::back::DataProcessorApplication
 * Created on: 11 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include "DataProcessorApplication.h"
#include "AbstractDataReceiver.h"
#include <mdcs/acq/StateMachineFactory.h>
#include <mdcs/cfg/IdentityProvider.h>
#include <mdcs/cfg/Constants.h>
#include <mdcs/cfg/CCfgUtils.ipp>
#include <mdcs/utl/PolicyTuple.hpp>
#include <mdcs/utl/Tuples.hpp>
#include <dhsm/Engine.h>
#include <utl/Factory.ipp>
#include <utl/Singleton.ipp>
#include <utl/Construct.hpp>
#include <stdexcept>

namespace mdcs
{
namespace acq
{
namespace back
{
using namespace dhsm;

// =============================================================================
// mdcs::acq::back::DataProcessorApplication class methods body
// -----------------------------------------------------------------------------

DataProcessorApplication::DataProcessorApplication(const Params& params, ApplicationLoader& loader)
	: ServerApplication(params, loader,
		LeafEngineCreator(),
		cfg::PROCESSOR_CTRL_SERVANT,
		cfg::PROCESSOR_SM_NAME)
{
}

DataProcessorApplication::~DataProcessorApplication()
{
}

void DataProcessorApplication::processFrame(const uint8_t* buffer, uint32_t nBytes)
{
	BaseState* state =
		dynamic_cast<BaseState*>(
			server().engine()->currentStateMachine().currentStateData()
		);
	if(state != 0)
	{
		state->processFrame(buffer, nBytes);
	}
	else
	{
		throw std::runtime_error("invalid state data");
	}
}

std::auto_ptr<dhsm::StateMachine> DataProcessorApplication::createStateMachine()
{
	StateMachinePtr acqSm = createBaseStateMachine();
	State& configured = acqSm->findState("CONFIGURED");
	configured.setStateDataType<Configured>();
	configured.enterActions().add(&Configured::configure);

	State& started = configured.stateMachine().findState("STARTED");
	started.setStateDataType<Started>();
	started.enterActions().add(&Started::start);
	return acqSm;
}

static dhsm::StateMachine* stateMachineFactoryWrapper()
{
	return DataProcessorApplication::createStateMachine().release();
}

REGISTER_FACTORY(StateMachineFactory, cfg::PROCESSOR_SM_NAME, stateMachineFactoryWrapper)
REGISTER_FACTORY(ApplicationFactory, cfg::PROCESSOR_APPNAME, ::utl::Construct<DataProcessorApplication::Params*>())

// =============================================================================
// mdcs::acq::back::DataProcessorApplication::BaseState class methods body
// -----------------------------------------------------------------------------

DataProcessorApplication::BaseState::~BaseState()
{
}

void DataProcessorApplication::BaseState::processFrame(const uint8_t* /*buffer*/, uint32_t /*nBytes*/)
{
	// @TODO: throw exception
	throw std::runtime_error("cannot process frames in this state!");
}

// =============================================================================
// mdcs::acq::back::DataProcessorApplication::Configured class methods body
// -----------------------------------------------------------------------------

DataProcessorApplication::Configured::Configured(const StateData& base)
	: ConfiguredPolicies<Policies>(base)
{
}

DataProcessorApplication::Configured::~Configured()
{
}

// =============================================================================
// mdcs::acq::back::DataProcessorApplication::Started class methods body
// -----------------------------------------------------------------------------

DataProcessorApplication::Started::Started(const StateData& base)
	: StartedPolicies<Policies>(base)
{
}

DataProcessorApplication::Started::~Started()
{
}

void DataProcessorApplication::Started::processFrame(const uint8_t* buffer, uint32_t nBytes)
{
	processorPolicy().processFrame(buffer, nBytes);
}

AbstractDataProcessor& DataProcessorApplication::Started::processorPolicy()
{
	return *policies_.get<1>();
}

} /* namespace back */
} /* namespace acq */
} /* namespace mdcs */
