/* =============================================================================
 * DataProcessorApplication.h
 * -----------------------------------------------------------------------------
 * class mdcs::acq::back::DataProcessorApplication
 * Created on: 11 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef mdcs_acq_back_DataProcessorApplication_h_INCLUDED
#define mdcs_acq_back_DataProcessorApplication_h_INCLUDED

#include "AbstractDataProcessor.h"
#include "AbstractDataReceiver.h"
#include <mdcs/acq/PolicyStates.hpp>
#include <mdcs/acq/ServerApplication.h>
#include <boost/tuple/tuple.hpp>
#include <boost/cstdint.hpp>
#include <string>

// forward declaration
namespace dhsm { class StateMachine; }

namespace mdcs
{
namespace acq
{
namespace back
{

class DataProcessorApplication : public ServerApplication
{
public:
	static std::auto_ptr<dhsm::StateMachine> createStateMachine();

	struct Params : ApplicationParamsHelper<DataProcessorApplication, ServerApplication>
	{
		template<class C> static void init(Params& obj, const C& cfg);
	};

	DataProcessorApplication(const Params& params, ApplicationLoader& loader);
	virtual ~DataProcessorApplication();

	void processFrame(const uint8_t* buffer, uint32_t nBytes);

	typedef mpl::vector<DataReceiver, DataProcessor> Policies;

	class BaseState;
	class Configured;
	class Started;
};

class DataProcessorApplication::BaseState
{
public:
	virtual ~BaseState();
	virtual void processFrame(const uint8_t* buffer, uint32_t nBytes);
};

class DataProcessorApplication::Configured : public ConfiguredPolicies<Policies>, public BaseState
{
public:
	Configured(const StateData& base);
	virtual ~Configured();

	template<class C> static void init(Configured& obj, const C& config);

private:
	std::string daqGroup;
};

class DataProcessorApplication::Started : public StartedPolicies<Policies>, public BaseState
{
public:
	Started(const StateData& base);
	virtual ~Started();

	void processFrame(const uint8_t* buffer, uint32_t nBytes);
protected:
	AbstractDataProcessor& processorPolicy();
};

} /* namespace back */
} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_back_DataProcessorApplication_h_INCLUDED */
