/**
 * @file FileStorage.h
 * @date 16 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_back_policy_FileStorage_h_INCLUDED
#define mdcs_acq_back_policy_FileStorage_h_INCLUDED

#include "../AbstractPipelineStage.h"
#include <mdcs/acq/mon/ManagedMetric.hpp>
#include <mdcs/acq/mon/Metric.h>
#include <mdcs/utl/Policy.hpp>
#include <fstream>

namespace mdcs
{
namespace acq
{
namespace back
{
namespace policy
{

BEGIN_POLICY(PipelineStage, FileStorage)

	class Params : public utl::BaseParamData<Policy>
	{
	public:
		Params();

		template<class C> static void init(Params& s, const C& config);

		bool storeData; ///< when false, no data will be written.
		bool limitFileSize; ///< indicates whether maximum file size shall be enforced.
		uint64_t maxFileSize; ///< the maximum size of written files.
		int seqDigitCount; ///< the number of digit of the sequence number suffix.
		std::string dataDir; ///< the full-path of the directory where to create the files.
		std::string filePrefix;
		std::string fileSuffix; ///< the extension of data files.

		static const unsigned DEFAULT_DIGIT_COUNT = 3;
	};
	class Setup : public utl::BaseSetupData<Policy>
	{
	public:
		Setup(ApplicationState& state, const Params& params);

	private:
		friend class FileStorage::RunImpl;

		mon::MetricPtr bytesWrittenMetric_;
		mon::MetricPtr eventsReceivedMetric_;
	};
	class RunImpl : public utl::BaseRunImpl<Policy>
	{
	public:
		RunImpl(ApplicationState& state, Setup& setup, const RunInfo& runInfo);
		~RunImpl();

		virtual void process(const uint8_t* buffer, uint32_t nBytes);

		std::string getCurrentDataFilePath();
	private:
		void beginFile();
		void endFile();

		void reportError(const char* description);

		static unsigned computeSequenceMax(unsigned digitCount);

		mon::MetricValue<int64_t> bytesWritten_;
		mon::MetricValue<int64_t> eventsReceived_;
		bool hasFailed;
		unsigned sequenceNumber; ///< the sequence number of the current file.
		uint64_t currentFileSize; ///< the size of the current file.
		std::string currentFilePath; ///< the full-path of the current file.
		std::ofstream currentFile; ///< the output stream to the current file.
	};

END_POLICY(FileStorage)

} /* namespace policy */
} /* namespace back */
} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_back_policy_FileStorage_h_INCLUDED */
