/**
 * @file AcqFrontCCfg.cpp
 * @date 25 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "AcqFrontCCfg.h"
#include "DataGeneratorApplication.h"
#include "DataControllerApplication.h"
#include "policy/IceDataSender.h"
#include <mdcs/cfg/CCfgConfig.h>
#include <mdcs/cfg/IdentityProvider.h>
#include <mdcs/cfg/CCfgUtils.ipp>
#include <CCfg/Object.h>

namespace mdcs
{
namespace acq
{
namespace front
{

namespace policy
{
typedef std::pair<CCfg::CConfig, CCfg::CConfig> CCfgPair;

template<>
void IceDataSender::Params::init(Params& obj, const CCfgPair& config)
{
	CCfg::CConfig senderCfg = config.first;
	CCfg::CConfig receiverCfg = config.second;
	obj.doSend          = senderCfg("doSend");
	obj.timeout_ms      = senderCfg("timeout");
	obj.maxEventSize_KB = senderCfg("maxEventSize");
	obj.collocated = receiverCfg("collocated");
	obj.nodeName   = receiverCfg.resolve().getAttribute()->getParent()->getParent()->getIndex();
}
}

template<>
void DataControllerApplication::Configured::init(Configured& obj, const cfg::CCfgConfig& config)
{
	CCfg::CConfig root = config.root();
	CCfg::CConfig self = root("DataController", cfg::IdentityProvider::instance().id().toString());

	ConfiguredPolicies<Policies>::init(obj, self);
}

template<>
void DataGeneratorApplication::Configured::init(Configured& obj, const cfg::CCfgConfig& config)
{
	CCfg::CConfig root = config.root();
	CCfg::CConfig self = root("DataGenerator", cfg::IdentityProvider::instance().id().toString());

	ConfiguredPolicies<Policies>::init(obj, self);
}

template<>
void DataControllerApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	ServerApplication::Params::init(p, subconfig);
}

template<>
void DataControllerApplication::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

template<>
void DataGeneratorApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	ServerApplication::Params::init(p, subconfig);
}

template<>
void DataGeneratorApplication::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}


CCfgInit::CCfgInit()
{
	cfg::Configurator::instance()
		.add(&policy::IceDataSender::Params::init<policy::CCfgPair>)
		.add(&DataGeneratorApplication::Configured::init<cfg::CCfgConfig>)
		.add(&DataControllerApplication::Configured::init<cfg::CCfgConfig>)
		.add(&DataControllerApplication::Params::init<CCfg::CConfig>)
		.add(&DataControllerApplication::Params::init<cfg::CCfgConfig>)
		.add(&DataGeneratorApplication::Params::init<CCfg::CConfig>)
		.add(&DataGeneratorApplication::Params::init<cfg::CCfgConfig>)
		;
}

} /* namespace front */
} /* namespace acq */

namespace utl
{
template<>
CCfg::CConfig getPolicyConfig(const PolicyParams<acq::front::DataController>&, CCfg::CConfig inConfig, const std::string& /*metaPolicyName*/, std::string& policyClass)
{
	policyClass = inConfig("policy").getStringValue();
	return inConfig(policyClass);
}

std::pair<CCfg::CConfig, CCfg::CConfig>
getPolicyConfig(const PolicyParams<acq::front::DataSender>&, CCfg::CConfig inConfig, const std::string& metaPolicyName, std::string& policyClass)
{
	std::string targetName = inConfig("target");
	CCfg::CConfig receiver = CCfg::CConfig(inConfig.getBaseAttribute()->getParent())
	                          ("DataProcessor", targetName)
	                          ("DataReceiver");
	std::string receiverPolicy = receiver("policy");
	policyClass = receiverPolicy;
	size_t index = policyClass.find("Receiver");
	policyClass.erase(index);
	policyClass.append("Sender");

	return std::make_pair(
			inConfig(metaPolicyName)(policyClass),
			receiver(receiverPolicy));
}

} /* namespace utl */
} /* namespace mdcs */
