/**
 * @file DataControllerApplication.cpp
 * @date 7 août 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "DataControllerApplication.h"
#include <mdcs/acq/StateMachineFactory.h>
#include <mdcs/cfg/IdentityProvider.h>
#include <mdcs/cfg/Constants.h>
#include <dhsm/StateMachine.h>
#include <utl/Factory.ipp>
#include <utl/Singleton.ipp>
#include <utl/Construct.hpp>

namespace mdcs
{
namespace acq
{
namespace front
{
using namespace dhsm;

// =============================================================================
// mdcs::acq::front::DataControllerApplication class methods body
// -----------------------------------------------------------------------------

DataControllerApplication::DataControllerApplication(const Params& params, ApplicationLoader& loader)
	: ServerApplication(params,
		loader,
		LeafEngineCreator(),
		cfg::CONTROLLER_CTRL_SERVANT,
		cfg::CONTROLLER_SM_NAME)
{
}

DataControllerApplication::~DataControllerApplication()
{
}

std::auto_ptr<dhsm::StateMachine> DataControllerApplication::createStateMachine()
{
	StateMachinePtr acqSm = createBaseStateMachine();
	State& configured = acqSm->findState("CONFIGURED");
	configured.setStateDataType<Configured>();
	configured.enterActions().add(&Configured::configure);

	State& started = configured.stateMachine().findState("STARTED");
	started.setStateDataType<Started>();
	started.enterActions().add(&Started::start);
	return acqSm;
}

static dhsm::StateMachine* stateMachineFactoryWrapper()
{
	return DataControllerApplication::createStateMachine().release();
}

REGISTER_FACTORY(StateMachineFactory, cfg::CONTROLLER_SM_NAME, stateMachineFactoryWrapper)
REGISTER_FACTORY(ApplicationFactory, cfg::CONTROLLER_APPNAME, ::utl::Construct<DataControllerApplication::Params*>())

// =============================================================================
// mdcs::acq::front::DataControllerApplication::Configured class methods body
// -----------------------------------------------------------------------------

DataControllerApplication::Configured::Configured(const StateData& base)
	: ConfiguredPolicies<Policies>(base)
{
}

DataControllerApplication::Configured::~Configured()
{
}

// =============================================================================
// mdcs::acq::front::DataControllerApplication::Started class methods body
// -----------------------------------------------------------------------------

DataControllerApplication::Started::Started(const StateData& base)
	: StartedPolicies<Policies>(base)
{
}

DataControllerApplication::Started::~Started()
{
}

} /* namespace front */
} /* namespace acq */
} /* namespace mdcs */
