/**
 * @file DataGeneratorApplication.h
 * @date 22 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_front_DataGeneratorApplication_h_INCLUDED
#define mdcs_acq_front_DataGeneratorApplication_h_INCLUDED

#include "AbstractDataProducer.h"
#include <mdcs/acq/front/AbstractDataSender.h>
#include <mdcs/acq/ServerApplication.h>
#include <mdcs/acq/PolicyStates.hpp>
#include <boost/scoped_ptr.hpp>
#include ICEUTIL_HEADER(Monitor.h)
#include ICEUTIL_HEADER(Mutex.h)
#include ICEUTIL_HEADER(Thread.h)

// forward declaration
namespace dhsm { class StateMachine; }

namespace mdcs
{
namespace acq
{
class AbstractInputBuffer;

namespace front
{

class DataGeneratorApplication : public ServerApplication
{
public:
	static std::auto_ptr<dhsm::StateMachine> createStateMachine();

	struct Params : ApplicationParamsHelper<DataGeneratorApplication, ServerApplication>
	{
		template<class C> static void init(Params& obj, const C& cfg);
	};

	DataGeneratorApplication(const Params& params, ApplicationLoader& loader);
	virtual ~DataGeneratorApplication();

	typedef mpl::vector<DataSender, DataProducer> Policies;

	class SenderThread;

	class Configured;
	class Started;
};

class DataGeneratorApplication::SenderThread : public IceUtil::Thread
{
public:
	SenderThread(AbstractDataSender& sender, AbstractDataProducer& producer_);

	void waitReady();
	void askStop();

private:
	void run();
	void signalReady();

	AbstractDataSender& sender_;
	AbstractDataProducer& producer_;
	boost::scoped_ptr<AbstractInputBuffer> buffer_;
	IceUtil::Monitor<IceUtil::Mutex> readyMonitor_;
	bool isReady_;
};

class DataGeneratorApplication::Configured : public ConfiguredPolicies<Policies>
{
public:
	Configured(const StateData& base);
	virtual ~Configured();

	template<class C> static void init(Configured& obj, const C& config);
};

class DataGeneratorApplication::Started : public StartedPolicies<Policies>
{
public:
	Started(const StateData& base);
	virtual ~Started();

protected:
	void doStart();

private:
	utl::PolicyRunImpl<DataSender>& dataSender() { return policies_.get<0>(); }
	utl::PolicyRunImpl<DataProducer>& dataProducer() { return policies_.get<1>(); }

	IceUtil::Handle<SenderThread> senderThread_;
	IceUtil::ThreadControl senderThreadCtrl_;
};

} /* namespace front */
} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_front_DataGeneratorApplication_h_INCLUDED */
