/**
 * @file IceDataSender.cpp
 * @date 18 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "IceDataSender.h"
#include "../AbstractInputBuffer.h"
#include <mdcs/acq/ServerApplication.h>
#include <mdcs/cfg/Constants.h>
#include <utl/io/Buffer.hpp>
#include <utl/Logging.h>
#include <sstream>

namespace mdcs
{
namespace acq
{
namespace front
{
namespace policy
{

using ::utl::io::ConstBuffer;

// =============================================================================
// mdcs::acq::front::policy::IceDataSender::Setup class methods body
// -----------------------------------------------------------------------------

IceDataSender::Setup::Setup(ApplicationState& state, const Params& params)
	: utl::BaseSetupData<Policy>(state, params),
	  buffer(new uint8_t[params.maxEventSize_KB * 1024])
{
	std::ostringstream stringProxy;
	stringProxy << cfg::PROCESSOR_FLOW_SERVANT << '@' << params.nodeName;
	target = IDataProcessorPrx::checkedCast(
		state.application().server().adapter()->getCommunicator()->stringToProxy(stringProxy.str())
			->ice_timeout(params.timeout_ms));
}

IceDataSender::Setup::~Setup()
{
}

// =============================================================================
// mdcs::acq::front::policy::IceDataSender::Setup class methods body
// -----------------------------------------------------------------------------

IceDataSender::RunImpl::RunImpl(ApplicationState& state, Setup& setup, const RunInfo& runInfo)
	: utl::BaseRunImpl<Policy>(state, setup, runInfo)
{
}

bool IceDataSender::RunImpl::sendData(AbstractInputBuffer& input)
{
	uint8_t* bufferPtr = setup.buffer.get();
	//TODO: get read timeout from configuration
	AbstractInputBuffer::Handle handle(input.read(1, 2000, false));
	while(handle->hasNext())
	{
		bool frameEnd;
		ConstBuffer chunk = handle->next(frameEnd);
		if(params.doSend)
		{
			if(bufferPtr + chunk.maxBytes > setup.buffer.get() + params.maxEventSize_KB * 1024)
			{
				LOG_WARN() << "oversized event !";
				break;
			}
			memcpy(bufferPtr, chunk.data, chunk.maxBytes);
			bufferPtr += chunk.maxBytes;
			if(frameEnd)
			{
				setup.target->processData(std::make_pair(setup.buffer.get(), bufferPtr));
				bufferPtr = setup.buffer.get();
			}
		}
	}
	AbstractInputBuffer::Status status = handle->getStatus();
	return status == AbstractInputBuffer::READ_OK or status == AbstractInputBuffer::TIMEOUT;
}

} /* namespace policy */
} /* namespace front */
} /* namespace acq */
} /* namespace mdcs */
