/**
 * @file ManagedMetric.hpp
 * @date 2 oct. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_mon_ManagedMetric_hpp_INCLUDED
#define mdcs_acq_mon_ManagedMetric_hpp_INCLUDED

#include <boost/date_time/posix_time/ptime.hpp>
#include <boost/intrusive/list.hpp>
#include <boost/function.hpp>
#include <boost/cstdint.hpp>

namespace mdcs
{
namespace acq
{
namespace mon
{
// forward declarations
class Metric;

class BaseManagedMetric : public boost::intrusive::list_base_hook<>
{
public:
	typedef boost::posix_time::ptime SampleTimestamp;

	BaseManagedMetric(Metric& metric, uint16_t freqDivisor);
	virtual ~BaseManagedMetric();

	void conditionalReportSample(const SampleTimestamp& defaultTimestamp);

	virtual void reportSample(const SampleTimestamp& defaultTimestamp) = 0;

protected:
	Metric& metric_;
	uint16_t freqDivisor_;
	uint16_t sampleOrdinal_;
};

template<typename T>
class ManagedMetric : public BaseManagedMetric
{
public:
	typedef boost::function<T (SampleTimestamp&)> SamplerFunc;

	ManagedMetric(Metric& metric, T& variable, uint16_t freqDivisor = 1);
	ManagedMetric(Metric& metric, const SamplerFunc& sampler, uint16_t freqDivisor = 1);
	virtual ~ManagedMetric();

	void reportSample(const SampleTimestamp& defaultTimestamp);

protected:
	static T varSampler(T& varRef, SampleTimestamp&);

	SamplerFunc sampler_;
};

template<typename T>
class MetricValue : public ManagedMetric<T>
{
public:
	MetricValue(Metric& metric, uint16_t freqDivisor = 1);
	MetricValue(T value, Metric& metric, uint16_t freqDivisor = 1);
	~MetricValue();

	operator T&() { return value_; }
	operator T() const { return value_; }

protected:
	T value_;
};

} /* namespace mon */
} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_mon_ManagedMetric_hpp_INCLUDED */
