/**
 * @file Metric.h
 * @date 2 oct. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_acq_mon_Metric_h_INCLUDED
#define mdcs_acq_mon_Metric_h_INCLUDED

#include <utl/io/Buffer.hpp>
#include <boost/date_time/posix_time/ptime.hpp>
#include <boost/utility.hpp>
#include <boost/cstdint.hpp>
#include <vector>
#include <string>
#include <memory>

namespace mdcs
{
namespace acq
{
namespace mon
{
// forward declarations
class MetricsCollector;

class AbstractMetric
{
public:
	typedef boost::posix_time::ptime SampleTimestamp;

	virtual ~AbstractMetric();

	virtual void reportSample(const int64_t& sample, const SampleTimestamp& time) = 0;
	virtual void reportSample(const double& sample, const SampleTimestamp& time) = 0;
	virtual void reportSample(const std::string& sample, const SampleTimestamp& time) = 0;

	virtual void reportSample(const std::vector<int64_t>& samples, const SampleTimestamp& time) = 0;
	virtual void reportSample(const std::vector<double>& samples, const SampleTimestamp& time) = 0;
	virtual void reportSample(const std::vector<std::string>& samples, const SampleTimestamp& time) = 0;

	virtual void reportSample(const ::utl::io::ConstBuffer& sample, const SampleTimestamp& time) = 0;
};

class Metric : public AbstractMetric, public boost::noncopyable
{
public:
	static const char META_METRIC_ID[];

	virtual ~Metric();

	MetricsCollector& collector() { return collector_; }
	const MetricsCollector& collector() const { return collector_; }
	const std::string& id() const { return id_; }

	static SampleTimestamp currentTimestamp();

protected:
	Metric(MetricsCollector& collector, const std::string& id);

	MetricsCollector& collector_;
	std::string id_;
};

typedef std::auto_ptr<Metric> MetricPtr;

} /* namespace mon */
} /* namespace acq */
} /* namespace mdcs */

#endif /* mdcs_acq_mon_Metric_h_INCLUDED */
