/* =============================================================================
 * AbstractConfig.h
 * -----------------------------------------------------------------------------
 * class mdcs::cfg::AbstractConfig
 * Created on: 25 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef mdcs_cfg_AbstractConfig_h_INCLUDED
#define mdcs_cfg_AbstractConfig_h_INCLUDED

#include <utl/Configurator.h>
#include <utl/Factory.hpp>
#include <utl/Singleton.hpp>

namespace mdcs
{
namespace cfg
{

class AbstractConfig
{
public:
	AbstractConfig();
	virtual ~AbstractConfig();

	virtual std::string encode() = 0;

	static void selectConfigFormat(const std::string& formatName);
};

typedef std::auto_ptr<AbstractConfig> ConfigPtr;

typedef AbstractConfig* ConfigFactoryMethod(const std::string&);
typedef ::utl::Singleton< ::utl::FactoryMap<std::string, ConfigFactoryMethod> > ConfigFormatRegistry;
typedef ::utl::Singleton< ::utl::Factory<ConfigFactoryMethod> > ConfigFactory;

typedef ::utl::GlobalConfigurator Configurator;

} /* namespace cfg */
} /* namespace mdcs */

#endif /* mdcs_cfg_AbstractConfig_h_INCLUDED */
