/* =============================================================================
 * CCfgConfig.cpp
 * -----------------------------------------------------------------------------
 * class mdcs::cfg::CCfgConfig
 * Created on: 25 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include "CCfgConfig.h"
#include "IdentityProvider.h"
#include <CCfg/Io/XmlFormat.h>
#include <CCfg/Io/BufferStorage.h>
#include <CCfg/Object.h>
#include <utl/Factory.ipp>
#include <utl/Singleton.ipp>

namespace mdcs
{
namespace cfg
{

using namespace CCfg;

CCfgConfig::CCfgConfig(std::auto_ptr<CCfg::Object>& config)
	: root_(config)
{
}

CCfgConfig::~CCfgConfig()
{
}

CCfg::CConfig CCfgConfig::applicationConfig() const
{
	CConfig cfg = root();
	CConfig app = cfg("Application", cfg::IdentityProvider::instance().id().type());
	Attribute* appInst = app("Instance", cfg::IdentityProvider::instance().id().num()).search();
	return (appInst != 0 ? CConfig(appInst) : app("Instance", CCfg::DEFAULT_OBJ_INDEX));
}

CCfg::CConfig CCfgConfig::nodeConfig(const std::string& nodeClass)
{
	CConfig cfg = root();
	return cfg(nodeClass, cfg::IdentityProvider::instance().id().toString());
}

std::string CCfgConfig::encode()
{
//TODO: if multiple formats are possible, we should use the same as the one used for decoding.
	return Io::BufferStorage::encode<Io::XmlFormat>(root_.get());
}

std::auto_ptr<CCfgConfig> CCfgConfig::createFromXml(const std::string& config)
{
	std::auto_ptr<Object> root =
		config.empty() ?
			std::auto_ptr<Object>(new Object) :
			Io::BufferStorage::decode<Io::XmlFormat>(config);
	return std::auto_ptr<CCfgConfig>(new CCfgConfig(root));
}

static CCfgConfig* createFromXmlWrapper(const std::string& config)
{
	return CCfgConfig::createFromXml(config).release();
}

} /* namespace cfg */
} /* namespace mdcs */

REGISTER_FACTORY(mdcs::cfg::ConfigFormatRegistry, "CCfg-Xml", &mdcs::cfg::createFromXmlWrapper)
