/**
 * @file CommandLineIdentityProvider.cpp
 * @date 8 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "CommandLineIdentityProvider.h"
#include <utl/Factory.ipp>
#include <utl/Singleton.ipp>
#include <utl/Construct.hpp>

namespace mdcs
{
namespace cfg
{

CommandLineIdentityProvider::CommandLineIdentityProvider()
{
}

CommandLineIdentityProvider::~CommandLineIdentityProvider()
{
}

void CommandLineIdentityProvider::init(const Ice::PropertiesPtr& properties)
{
	std::string idType = id_.type();
	std::string idNum = boost::lexical_cast<std::string>(id_.num());
	std::string fullId = properties->getProperty("Mordicus.Id");
	if(!fullId.empty())
	{
		size_t separatorIdx = fullId.find_last_of('_');
		if(separatorIdx != std::string::npos)
		{
			idNum = fullId.substr(separatorIdx + 1);
		}
		idType = fullId.substr(0, separatorIdx);
	}
	else
	{
		idType = properties->getPropertyWithDefault("Mordicus.Id.Type", idType);
		idNum = properties->getPropertyWithDefault("Mordicus.Id.Num", idNum);
	}

	if(idType.empty())
	{
		throw std::runtime_error("Empty identity type (use --Mordicus.Id.Type parameter to set it)");
	}
	if(idNum.empty())
	{
		throw std::runtime_error("Empty identity number (use --Mordicus.Id.Num parameter to set it)");
	}

	int ordinal;
	try
	{
		ordinal = boost::lexical_cast<int>(idNum);
	}
	catch(boost::bad_lexical_cast& e)
	{
		throw std::runtime_error("Invalid identity number");
	}
	id_.setType(idType);
	id_.setNum(ordinal);
}

CommandLineIdentityProvider::Init::Init()
{
	IdentityProviderFactory::instance().add("CommandLine", utl::Construct<CommandLineIdentityProvider*>());
}

} /* namespace cfg */
} /* namespace mdcs */
