/**
 * @file IdentityProvider.h
 * @date 8 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_cfg_IdentityProvider_h_INCLUDED
#define mdcs_cfg_IdentityProvider_h_INCLUDED

#include "NodeId.h"
#include <utl/Factory.hpp>
#include <utl/Singleton.hpp>
#include ICE_HEADER(Properties.h)
#include <boost/scoped_ptr.hpp>

namespace mdcs
{
namespace cfg
{

class AbstractIdentityProvider
{
public:
	virtual ~AbstractIdentityProvider();

	virtual void init(const Ice::PropertiesPtr& properties) = 0;

	void setType(const std::string& type) { id_.setType(type); }
	void setNum(int num) { id_.setNum(num); }

	const NodeId& id() const { return id_; }

protected:
	NodeId id_;
};

/**
 * This class is a singleton which determines the identity of the current process
 * in the system.
 */
class IdentityProviderBridge
{
public:
	void setProvider(std::auto_ptr<AbstractIdentityProvider> provider);

	void setDefaultProvider(const std::string& providerName) { defaultProvider = providerName; }
	void setDefaultType(const std::string& type) { defaultType = type; }

	void init(const Ice::PropertiesPtr& properties);

	const NodeId& id() const;

	friend class ::utl::Singleton<IdentityProviderBridge>;
private:
	IdentityProviderBridge();
	~IdentityProviderBridge();

	boost::scoped_ptr<AbstractIdentityProvider> impl;

	/// if this provider name is set, it will be used to instantiate the
	/// identity provider if no other name is specified in the command line.
	std::string defaultProvider;

	/// if this type name is set, it will be used to configure the identity if
	/// no other name is specified in the command line.
	std::string defaultType;
};

typedef ::utl::Singleton<IdentityProviderBridge> IdentityProvider;
typedef ::utl::Singleton< ::utl::FactoryMap<std::string, AbstractIdentityProvider*()> > IdentityProviderFactory;

} /* namespace cfg */
} /* namespace mdcs */

#endif /* mdcs_cfg_IdentityProvider_h_INCLUDED */
