/**
 * @file CtrlCCfg.cpp
 * @date 2 avr. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "CtrlCCfg.h"
#include "RunControlApplication.h"
#include "FileRunRepository.h"
#include <mdcs/cfg/CCfgUtils.ipp>
#include <mdcs/cfg/CCfgConfig.h>
#include <mdcs/cfg/Constants.h>
#include <CCfg/Object.h>
#include <utl/Logging.h>

namespace mdcs
{
namespace ctrl
{

using namespace CCfg;
using namespace acq;

template<>
void RunControlApplication::Connected::init(Connected& obj, const cfg::CCfgConfig& config)
{
	CConfig cfg = config.root();

	cfg::createPolicyParams(obj.runRepositoryParams_, cfg("RunRepository"));

	CConfig::Iterator smIt = cfg.iterate("Statemachine");
	while(smIt.hasNext())
	{
		CConfig statemachine = smIt.next();
		std::string smId = statemachine.getIndex();
		if(smId != "*" and statemachine("isActive").getBoolValue() == true)
		{
			LOG_DEBUG() << "configuring " << statemachine.getPath();
			const StateMachineServer* server;
			if(smId == cfg::PROCESSOR_CTRL_SERVANT)
			{
				server = &obj.application().server();
			}
			else
			{
				server = &obj.addStateMachine(smId, statemachine("class"));
			}
			CConfig::Iterator nodeIt = statemachine.iterate("Node");
			while(nodeIt.hasNext())
			{
				CConfig node = nodeIt.next();
				if(node.getIndex() != "*" and node("isActive").getBoolValue() == true)
				{
					std::string proxy(smId);
					proxy += '@';
					proxy += node.getIndex();
					LOG_DEBUG() << "connecting to " << proxy;
					obj.connectServant(*server, proxy);
				}
			}
		}
	}
}

template<>
void RunControlApplication::Configured::init(RunControlApplication::Configured& /*obj*/, const cfg::CCfgConfig& /*config*/)
{

}

template<>
void RunControlApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	ServerApplication::Params::init(p, subconfig);
}

template<>
void RunControlApplication::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

template<>
void FileRunRepository::Params::init(Params& params, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig config = subconfig;
	params.directory     = config("directory").getStringValue();
	params.siteName      = config("siteName").getStringValue();
	params.cfgFilePrefix = config("cfgFilePrefix").getStringValue();
	params.seqDigitCount = config("seqDigitCount");
}

CCfgInit::CCfgInit()
{
	cfg::Configurator::instance()
		.add( &RunControlApplication::Connected::init<cfg::CCfgConfig>)
		.add(&RunControlApplication::Configured::init<cfg::CCfgConfig>)
		.add(&RunControlApplication::Params::init<cfg::CCfgConfig>)
		.add(&RunControlApplication::Params::init<CCfg::CConfig>)
		.add(&FileRunRepository::Params::init<CCfg::CConfig>)
		;
}


} /* namespace ctrl */
} /* namespace mdcs */
