/* =============================================================================
 * RunControlApplication.h
 * -----------------------------------------------------------------------------
 * class mdcs::ctrl::RunControlApplication
 * Created on: 28 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#ifndef mdcs_ctrl_RunControlApplication_h_INCLUDED
#define mdcs_ctrl_RunControlApplication_h_INCLUDED

#include "RunConfigQueue.h"
#include <mdcs/acq/mon/Metric.h>
#include <mdcs/acq/ServerApplication.h>
#include <mdcs/acq/StateMachineFactory.h>
#include <dhsm/StateData.h>
#include <boost/ptr_container/ptr_map.hpp>
#include <boost/scoped_ptr.hpp>

// forward declaration
namespace dhsm { class StateMachine; }

namespace mdcs
{
namespace ctrl
{
class AbstractRunRepository;
class AbstractRunRepositoryParams;

class RunControlApplication : public acq::ServerApplication
{
public:
	struct Params : acq::ApplicationParamsHelper<RunControlApplication, acq::ServerApplication>
	{
		template<class C> static void init(Params& obj, const C& cfg);
	};

	RunControlApplication(const Params& params, acq::ApplicationLoader& loader);
	~RunControlApplication();

	void start();

	static std::auto_ptr<dhsm::StateMachine> createStateMachine();

	class ConfigHolder;

	class Connected;
	class Configured;
	class Started;

	acq::StateMachineServer& compositeServer() { return compositeServer_; }
	IRunConfigQueue& configQueue() { return *configQueue_; }
	AbstractRunRepository& runRepository();
	acq::mon::Metric& runInfoMetric() { return *runInfoMetric_; }

	void resetCompositeServer();

private:
	class BranchEngineCreator;
	class RootEngineCreator;
	class RootEngineServant;

	acq::StateMachineServer compositeServer_;
	IRunConfigQueuePtr configQueue_;
	acq::mon::MetricPtr runInfoMetric_;
	Ice::ObjectAdapterPtr configAdapter_;
};

class RunControlApplication::ConfigHolder
{
public:
	void setConfig(const std::string& cfg);
	const std::string& config() const;

private:
	std::string config_;
};

class RunControlApplication::Connected : public acq::ConfiguredState, public ConfigHolder
{
public:
	Connected(const StateData& base);
	~Connected();

	typedef boost::ptr_map<std::string, acq::StateMachineServer> StateMachineSet;

	acq::StateMachineServer& addStateMachine(const std::string& id, const std::string& nameInFactory);
	AbstractRunRepository& runRepository();

	template<class C> static void init(Connected& obj, const C& config);

protected:
	void apply();

private:
	void connectServant(const acq::StateMachineServer& server, const std::string& proxy);

	boost::scoped_ptr<AbstractRunRepositoryParams> runRepositoryParams_;
	boost::scoped_ptr<AbstractRunRepository> runRepository_;
	StateMachineSet stateMachines_;
};

class RunControlApplication::Configured : public acq::ConfiguredState, public ConfigHolder
{
public:
	Configured(const StateData& base);

	template<class C> static void init(Configured&, const C&);

protected:
	void apply();
};

class RunControlApplication::Started : public acq::ApplicationState
{
public:
	Started(const StateData& base);
	void recordRun(const std::string& runCfg);
	Connected& connectedCtx();
	Configured& configuredCtx();

private:
	void reportNewRun(const acq::RunInfo& runInfo);
};

} /* namespace ctrl */
} /* namespace mdcs */

#endif /* mdcs_ctrl_RunControlApplication_h_INCLUDED */
