/**
 * @file GuiApplication.h
 * @date 20 janv. 2014
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_gui_GuiApplication_h_INCLUDED
#define mdcs_gui_GuiApplication_h_INCLUDED

#include "ConfigSetMaster.h"
#include <mdcs/acq/Application.h>
#include <CCfg/Model/ConfigSetModel.h>
#include <QApplication>

class QMainWindow;
class QSettings;

namespace mdcs
{
// forward declarations
namespace ctrl { namespace mon { class MetricsMonitor; } }

namespace gui
{
// forward declarations
namespace sm { class StateMachineModel; }

class GuiApplication : public acq::Application
{
public:
	class Params;
	class Descriptor;

	GuiApplication(const Params& params, acq::ApplicationLoader& loader, const Descriptor& descriptor);
	virtual ~GuiApplication();

	void mainLoop();

	virtual QMainWindow& mainWindow() = 0;

	ctrl::mon::MetricsMonitor& monitor() { return *metricsMonitor_; }

protected:
	static void saveWindowShape(const QMainWindow& window);
	static void restoreWindowShape(QMainWindow& window);

private:
	boost::scoped_ptr<ctrl::mon::MetricsMonitor> metricsMonitor_;
	QApplication app;
};

class GuiApplication::Params : public acq::ApplicationParams
{
public:
	template<class C> static void init(Params& p, const C& config);
	virtual ~Params();

	std::string stateMachineName;
};

class GuiApplication::Descriptor
{
public:
	Descriptor(const char* organizationName,
		const char* organizationDomain,
		const char* applicationName,
		const char* applicationVersion);

	QString organizationName;
	QString organizationDomain;
	QString applicationName;
	QString applicationVersion;
};

class MasterGuiApplication : public GuiApplication
{
public:
	class Params;

	MasterGuiApplication(const Params& params, acq::ApplicationLoader& loader, const Descriptor& descriptor);
	virtual ~MasterGuiApplication();

protected:
	boost::scoped_ptr<sm::StateMachineModel> stateMachineModel_;
	boost::scoped_ptr<CCfg::Model::ConfigSetModel::Params> configSetParams_;
	CCfg::Model::ConfigSetModel configSetModel_;
	boost::scoped_ptr<ConfigSetMaster> configSetMaster_;

private:
	CCfg::Model::ConfigSetModel::Persistency configSetModelPersistency_;
};

class MasterGuiApplication::Params : public GuiApplication::Params
{
public:
	template<class C> static void init(Params& p, const C& config);
	virtual ~Params();

	std::string configSetTemplate;
};

} /* namespace gui */
} /* namespace mdcs */

#endif /* mdcs_gui_GuiApplication_h_INCLUDED */
