/**
 * @file GuiCCfg.cpp
 * @date 27 janv. 2014
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "GuiCCfg.h"
#include "MordicusRunControl.h"
#include "GuiApplication.h"
#include <mdcs/cfg/CCfgConfig.h>
#include <utl/Logging.h>

namespace mdcs
{
namespace gui
{

template<>
void GuiApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig cfg = subconfig;
	p.stateMachineName = cfg("stateMachine").getStringValue();
	ApplicationParams::init(p, subconfig);
}

template<>
void GuiApplication::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

template<>
void MasterGuiApplication::Params::init(Params& p, const CCfg::CConfig& subconfig)
{
	CCfg::CConfig cfg = subconfig;
	p.configSetTemplate = cfg("configSet").getStringValue();
	GuiApplication::Params::init(p, subconfig);
}

template<>
void MordicusRunControl::Params::init(Params& p, const CCfg::CConfig& instance)
{
	MasterGuiApplication::Params::init(p, instance);
}

template<>
void MordicusRunControl::Params::init(Params& p, const cfg::CCfgConfig& config)
{
	init(p, config.applicationConfig());
}

CCfgInit::CCfgInit()
{
	cfg::Configurator::instance()
		.add(&GuiApplication::Params::init<cfg::CCfgConfig>)
		.add(&GuiApplication::Params::init<CCfg::CConfig>)
		.add(&MasterGuiApplication::Params::init<CCfg::CConfig>)
		.add(&MordicusRunControl::Params::init<CCfg::CConfig>)
		.add(&MordicusRunControl::Params::init<cfg::CCfgConfig>)
		;
}

} /* namespace gui */
} /* namespace mdcs */
