/**
 * @file SignaledMetric.cpp
 * @date 7 oct. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#include "SignaledMetric.h"
#include <QMetaType>

namespace mdcs
{
namespace gui
{
namespace mon
{

BaseSignaledMetric::BaseSignaledMetric()
{
}

BaseSignaledMetric::~BaseSignaledMetric()
{
}

void BaseSignaledMetric::reportImpl(int64_t sample, const SampleTimestamp& time)
{
	emit report(static_cast<qint64>(sample), time);
}

void BaseSignaledMetric::reportImpl(double sample, const SampleTimestamp& time)
{
	emit report(sample, time);
}

void BaseSignaledMetric::reportImpl(const std::string& sample, const SampleTimestamp& time)
{
	emit report(QString::fromStdString(sample), time);
}

void BaseSignaledMetric::reportImpl(const std::vector<std::string>& sample, const SampleTimestamp& time)
{
	QStringList list;
	for(size_t i = 0; i < sample.size(); ++i)
	{
		list.push_back(QString::fromStdString(sample[i]));
	}
	emit report(list, time);
}

void BaseSignaledMetric::reportImpl(const ::utl::io::ConstBuffer& sample, const SampleTimestamp& time)
{
	emit report(QByteArray(reinterpret_cast<const char*>(sample.data), sample.maxBytes), time);
}

const char* BaseSignaledMetric::getSignal(const SignaledMetric<int64_t>*)
{
	return SIGNAL(report(qint64, const SampleTimestamp&));
}

const char* BaseSignaledMetric::getSignal(const SignaledMetric<double>*)
{
	return SIGNAL(report(double, const SampleTimestamp&));
}

const char* BaseSignaledMetric::getSignal(const SignaledMetric<std::string>*)
{
	return SIGNAL(report(const QString&, const SampleTimestamp&));
}

const char* BaseSignaledMetric::getSignal(const SignaledMetric< ::utl::io::ConstBuffer>*)
{
	return SIGNAL(report(const QByteArray&, const SampleTimestamp&));
}

} /* namespace mon */
} /* namespace gui */
} /* namespace mdcs */

typedef mdcs::acq::mon::Metric::SampleTimestamp SampleTimestamp;
Q_DECLARE_METATYPE(SampleTimestamp)
static const int sampleTimestampRegistered = qRegisterMetaType<SampleTimestamp>();
