/**
 * @file SignaledMetric.h
 * @date 7 oct. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_gui_mon_SignaledMetric_h_INCLUDED
#define mdcs_gui_mon_SignaledMetric_h_INCLUDED

#include <mdcs/ctrl/mon/MetricCallback.h>
#include <utl/Logging.h>
#include <QByteArray>
#include <QStringList>
#include <QObject>

namespace mdcs
{
namespace gui
{
namespace mon
{
// forward declarations
template<class T> class SignaledMetric;

class BaseSignaledMetric : public QObject, public ctrl::mon::BaseMetricCallback
{
	Q_OBJECT
public:
	BaseSignaledMetric();
	virtual ~BaseSignaledMetric();

signals:
	void report(qint64 sample, const SampleTimestamp& time);
	void report(double sample, const SampleTimestamp& time);
	void report(const QString& sample, const SampleTimestamp& time);
	void report(const QStringList& sample, const SampleTimestamp& time);
	void report(const QByteArray& sample, const SampleTimestamp& time);

protected:
	void reportImpl(int64_t sample, const SampleTimestamp& time);
	void reportImpl(double sample, const SampleTimestamp& time);
	void reportImpl(const std::string& sample, const SampleTimestamp& time);
	void reportImpl(const std::vector<std::string>& sample, const SampleTimestamp& time);
	void reportImpl(const ::utl::io::ConstBuffer& sample, const SampleTimestamp& time);

	template<class U>
	void reportImpl(const std::vector<U>& sample, const SampleTimestamp& time)
	{
		QStringList list;
		for(size_t i = 0; i < sample.size(); ++i)
		{
			list.push_back(QString::number(sample[i]));
		}
		emit report(list, time);
	}

	static const char* getSignal(const SignaledMetric<int64_t>*);
	static const char* getSignal(const SignaledMetric<double>*);
	static const char* getSignal(const SignaledMetric<std::string>*);
	static const char* getSignal(const SignaledMetric< ::utl::io::ConstBuffer>*);

	template<class T>
	static const char* getSignal(const SignaledMetric<std::vector<T> >*)
	{
		return SIGNAL(report(const QStringList&, const SampleTimestamp&));
	}
};

template<class T>
class SignaledMetric : public BaseSignaledMetric
{
public:
	SignaledMetric(QObject* receiver, const char* slt)
	{
		connect(this, this->getSignal(this), receiver, slt, Qt::QueuedConnection);
	}
	~SignaledMetric() {}

	void reportSample(const T& sample, const SampleTimestamp& time)
	{
		reportImpl(sample, time);
	}
};

} /* namespace mon */
} /* namespace gui */
} /* namespace mdcs */

#endif /* mdcs_gui_mon_SignaledMetric_h_INCLUDED */
