/**
 * @file MasterStateMachineModel.h
 * @date 5 fev 2014
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_gui_sm_MasterStateMachineModel_h_INCLUDED
#define mdcs_gui_sm_MasterStateMachineModel_h_INCLUDED

#include "StateMachineModel.h"
#include ICE_HEADER(Communicator.h)
#include <QThread>
#include <boost/scoped_ptr.hpp>
#include <memory>

namespace dhsm { class StateMachine; }

namespace mdcs
{
namespace gui
{
namespace sm
{
class StateMachineWidget;

class MasterStateMachineModel : public StateMachineModel
{
	Q_OBJECT
public:
	class AsyncModel;
	class Thread;

	MasterStateMachineModel(std::auto_ptr<dhsm::StateMachine> sm, const Ice::CommunicatorPtr& ic);
	~MasterStateMachineModel();

	void doGetReady();

	dhsm::Engine& engine() { return *engine_; }

signals:
	void stepEvent(const QString& event, const QString& param);
	void reinit();

public slots:
	void onStepEvent(const QString& event, const QString& param = QString());
	void onReinit();

private:
	boost::scoped_ptr<dhsm::Engine> engine_;
	boost::scoped_ptr<Thread> smThread_;
};

class MasterStateMachineModel::AsyncModel : public StateMachineModel
{
	Q_OBJECT
public:
	explicit AsyncModel(StateMachineModel& syncModel);
	~AsyncModel();

	dhsm::Engine& engine() { return engine_; }
	void doGetReady();

public slots:
	void onStepEvent(const QString& event, const QString& param = QString());
	void onReinit();

private:
	dhsm::Engine& engine_;
};

/**
 * This class implements a thread running a state-machine engine.
 */
class MasterStateMachineModel::Thread : public QThread
{
	Q_OBJECT
public:
	explicit Thread(StateMachineModel& syncModel);
	~Thread();

	void stop();

private:
	void run();

	StateMachineModel& syncModel_;
};

} /* namespace sm */
} /* namespace gui */
} /* namespace mdcs */

#endif /* mdcs_gui_sm_MasterStateMachineModel_h_INCLUDED */
