/**
 * @file CompositePolicy.hpp
 * @date 29 juil. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_utl_CompositePolicy_hpp_INCLUDED
#define mdcs_utl_CompositePolicy_hpp_INCLUDED

#include "Policy.hpp"
#include "PolicyTuple.hpp"
#include "Tuples.hpp"

namespace mdcs
{
namespace utl
{

template<class Policy, class SubPoliciesVector>
class CompositeRunImpl : public BaseRunImpl<Policy>
{
public:
	typedef typename PolicyTuple<SubPoliciesVector, RunImplHolder>::type PolicyRunImplTuple;
	typedef typename Policy::SetupData SetupData;
	typedef typename Policy::Context   Context;
	typedef typename Policy::RunInfo   RunInfo;

	CompositeRunImpl(Context& context, SetupData& s, const RunInfo& i)
		: BaseRunImpl<Policy>(context, s, i)
	{
		tuple_binary_apply(nestedPolicies_, s.nestedPoliciesSetup(), PolicyTupleFunc::create(context, i));
	}

	PolicyRunImplTuple& nestedPolicies() { return nestedPolicies_; }

protected:
	PolicyRunImplTuple nestedPolicies_;
};

template<class Policy, class SubPoliciesVector>
class CompositeSetupData : public BaseSetupData<Policy>
{
public:
	typedef typename PolicyTuple<SubPoliciesVector, SetupHolder>::type PolicySetupTuple;
	typedef typename Policy::ParamData  ParamData;
	typedef typename Policy::Context    Context;

	CompositeSetupData(Context& context, const ParamData& p)
		: BaseSetupData<Policy>(context, p)
	{
		tuple_binary_apply(nestedPoliciesSetup_, p.nestedPoliciesParams(), PolicyTupleFunc::create(context));
	}

	virtual ~CompositeSetupData() {}

	PolicySetupTuple& nestedPoliciesSetup() { return nestedPoliciesSetup_; }

protected:
	PolicySetupTuple nestedPoliciesSetup_;
};

template<class Policy, class SubPoliciesVector>
class CompositeParamData : public BaseParamData<Policy>
{
public:
	typedef typename PolicyTuple<SubPoliciesVector, ParamsHolder>::type PolicyParamsTuple;

	virtual ~CompositeParamData() {}

	template<class C> static void init(CompositeParamData& p, const C& config)
	{
		tuple_unary_apply(p.nestedPoliciesParams_, PolicyTupleFunc::init(config));
	}

	const PolicyParamsTuple& nestedPoliciesParams() const { return nestedPoliciesParams_; }

protected:
	PolicyParamsTuple nestedPoliciesParams_;
};

} /* namespace utl */
} /* namespace mdcs */

#endif /* mdcs_utl_CompositePolicy_hpp_INCLUDED */
