/* =============================================================================
 * @file ConsoleLoggingBackend.cpp
 * @created 8 oct. 2012
 * @author fchateau
 * ---------------------------------------------------------------------------------------------------------------------
 * class mdcs::utl::ConsoleLoggingBackend
 * Created on:  8 oct. 2012 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Frederic Chateau (frederic.chateau@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#ifndef mdcs_utl_ConsoleLoggingBackend_h_INCLUDED
#define mdcs_utl_ConsoleLoggingBackend_h_INCLUDED

#include <utl/Logging.h>

namespace mdcs
{
namespace utl
{
/**
 * This is a log backend that outputs logs on stdout or stderr (depending on
 * the severity), and decorates the log with a prompt denoting each level of
 * severity.
 */
class ConsoleLoggingBackend : public ::utl::AbstractLoggingBackend
{
public:
	ConsoleLoggingBackend();
	virtual ~ConsoleLoggingBackend();

	void setIdentity(const std::string& id) { identity = id; }

	std::ostream& beginLog(::utl::SeverityLevel level, const char* source, unsigned line);
	void endLog();

private:
	std::ostream* stream;
	std::string identity;
};

} /* namespace utl */
} /* namespace mdcs */

#endif /* mdcs_utl_ConsoleLoggingBackend_h_INCLUDED */
