/**
 * @file Tuples.hpp
 * @date 18 juil. 2013
 * @author fchateau
 * @note
 * SVN tag: $Id$
 *
 * Contributors: Frederic Chateau
 *
 * This file is part of the Mordicus software project.
 *
 * @copyright © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 *
 * @par FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 *
 * @par COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 *
 */

#ifndef mdcs_utl_Tuples_hpp_INCLUDED
#define mdcs_utl_Tuples_hpp_INCLUDED

#include <boost/static_assert.hpp>
#include <boost/tuple/tuple.hpp>

namespace mdcs
{
namespace utl
{

template<class T, int I, class F>
struct tuple_unary_apply_impl
{
	void operator()(T& tuple, F& f)
	{
		tuple_unary_apply_impl<T, I - 1, F>()(tuple, f);
		f(boost::tuples::get<I - 1>(tuple));
	}
};

template<class T, class F>
struct tuple_unary_apply_impl<T, 0, F>
{
	void operator()(T&, F&)
	{
	}
};

template<class T, class F>
void tuple_unary_apply(T& tuple, F f)
{
	tuple_unary_apply_impl<T, boost::tuples::length<T>::value, F>()(tuple, f);
}


template<class T1, class T2, int I, class F>
struct tuple_binary_apply_impl
{
	void operator()(T1& t1, T2& t2, F& f)
	{
		tuple_binary_apply_impl<T1, T2, I - 1, F>()(t1, t2, f);
		f(boost::tuples::get<I - 1>(t1), boost::tuples::get<I - 1>(t2));
	}
};

template<class T1, class T2, class F>
struct tuple_binary_apply_impl<T1, T2, 0, F>
{
	void operator()(T1&, T2&, F&)
	{
	}
};

template<class T1, class T2, class F>
void tuple_binary_apply(T1& t1, T2& t2, F f)
{
	BOOST_STATIC_ASSERT(boost::tuples::length<T1>::value == boost::tuples::length<T2>::value);
	tuple_binary_apply_impl<T1, T2, boost::tuples::length<T1>::value, F>()(t1, t2, f);
}

} /* namespace utl */
} /* namespace mdcs */

#endif /* mdcs_utl_Tuples_hpp_INCLUDED */
