/**
 * @file TypeInfo.hpp
 * @created Dec 18, 2012
 * @author anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */


#ifndef mdcs_utl_TypeInfo_hpp_INCLUDED
#define mdcs_utl_TypeInfo_hpp_INCLUDED

#include "utl/BitSet.hpp"
#include <boost/cstdint.hpp>
#include <boost/lexical_cast.hpp>
#include <string>
#include <typeinfo>

namespace mdcs
{
namespace utl
{

template <typename T>
struct TypeInfo
{
	static const std::string name() { return typeid(T).name(); }
};


template <>
struct TypeInfo<int8_t>
{
	static const std::string name() { return std::string("int8"); }
};

template <>
struct TypeInfo<uint8_t>
{
	static const std::string name() { return std::string("uint8"); }
};

template <>
struct TypeInfo<int16_t>
{
	static const std::string name() { return std::string("int16"); }
};

template <>
struct TypeInfo<uint16_t>
{
	static const std::string name() { return std::string("uint16"); }
};

template <>
struct TypeInfo<int32_t>
{
	static const std::string name() { return std::string("int32"); }
};

template <>
struct TypeInfo<uint32_t>
{
	static const std::string name() { return std::string("uint32"); }
};

template <>
struct TypeInfo<int64_t>
{
	static const std::string name() { return std::string("int64"); }
};

template <>
struct TypeInfo<uint64_t>
{
	static const std::string name() { return std::string("uint64"); }
};

template < ::utl::Signedness S >
struct TypeInfo< ::utl::RtBitSet<S> >
{
	static const std::string name() { return std::string(S == ::utl::SIGNED ? "" : "u") + "bitset"; }
};

template < size_t N, ::utl::Signedness S >
struct TypeInfo< ::utl::BitSet<N,S> >
{
	static const std::string name()
	{
		return TypeInfo< ::utl::RtBitSet<S> >::name() + '<' + boost::lexical_cast<std::string>(N) + '>';
	}
};

} // namespace utl
} // namespace mdcs


#endif /* mdcs_utl_TypeInfo_hpp_INCLUDED */
