/* =============================================================================
 * DataProcessorTest.cpp
 * -----------------------------------------------------------------------------
 * Created on: 15 mars 2013 at Irfu/Sedi/Lilas, CEA Saclay, F-91191, France
 * Author: fchateau
 * -----------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * -----------------------------------------------------------------------------
 * Contributors: fchateau
 * -----------------------------------------------------------------------------
 * This file is part of the Mordicus software project.
 * -----------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding
 * by the rules of distribution of free software. You can use, modify and/or
 * redistribute the software under the terms of the CeCILL license as circulated
 * by CEA, CNRS and INRIA at the following URL: "http://www.cecill.info".
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability. In this
 * respect, the user's attention is drawn to the risks associated with loading,
 * using, modifying and/or developing or reproducing the software by the user in
 * light of its specific status of free software, that may mean that it is
 * complicated to manipulate, and that also therefore means that it is reserved
 * for developers and experienced professionals having in-depth computer
 * knowledge. Users are therefore encouraged to load and test the software's
 * suitability as regards their requirements in conditions enabling the security
 * of their systems and/or data to be ensured and, more generally, to use and
 * operate it in the same conditions as regards security.
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * -----------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for
 * commercial purposes. For this you will need to negotiate a specific contract
 * with a legal representative of CEA.
 * =============================================================================
 */

#include <mdcs/acq/back/AcqBackCCfg.h>
#include <mdcs/acq/back/DataProcessorApplication.h>
#include <mdcs/acq/StateMachineFactory.h>
#include <mdcs/cfg/AbstractConfig.h>
#include <dhsm/Engine.h>
#include <dhsm/StateMachine.h>
#include <boost/test/auto_unit_test.hpp>

using namespace dhsm;
using namespace mdcs::acq;
using namespace mdcs::acq::back;

struct DataProcessorFixture
{
	DataProcessorFixture()
		: engine()
	{
		mdcs::cfg::AbstractConfig::selectConfigFormat("CCfg-Xml");
		StateMachinePtr sm = StateMachineFactory::instance()["Router"].create();
		{
			State& configured = sm->findState("CONFIGURED");
			configured.enterActions().clear();
		}
		stateMachine = sm.get();
		engine.setStateMachine(sm);
	}
	~DataProcessorFixture()
	{
	}

	Engine engine;
	StateMachine* stateMachine;
};

BOOST_FIXTURE_TEST_SUITE(mdcs_acq_DataProcessor, DataProcessorFixture)

BOOST_AUTO_TEST_CASE(test_initial_state)
{
	BOOST_CHECK(not engine.isInitiated(*stateMachine));
}

BOOST_AUTO_TEST_CASE(test_sm_start)
{
	Engine::ProcessingResult result = engine.start();
	BOOST_CHECK_EQUAL(result, Engine::Processed);
	BOOST_CHECK(engine.isInitiated(*stateMachine));
	BOOST_CHECK_EQUAL(engine.currentState().label(), "STANDBY");
	BOOST_CHECK_EQUAL(engine.currentStateMachine().parentCurrentStateData(), static_cast<void*>(0));
}

BOOST_AUTO_TEST_CASE(test_scenario1)
{
	engine.start();

	Engine::ProcessingResult configureResult = engine.step(Message<std::string>("configure", "<Config><toto>42</toto></Config>"));
	BOOST_CHECK_EQUAL(configureResult, Engine::Processed);
	BOOST_CHECK_EQUAL(engine.currentState().label(), "READY");
	StateData* sd = engine.currentStateMachine().parentCurrentStateData();
	BOOST_CHECK(sd != 0);
	DataProcessorApplication::Configured* configuredData =
		dynamic_cast<DataProcessorApplication::Configured*>(engine.currentStateMachine().parentCurrentStateData());
	BOOST_CHECK(configuredData != 0);
}

BOOST_AUTO_TEST_SUITE_END()
