/**
 * @file BitSet.hpp
 * @created Nov 27, 2012
 * @author anvar
 * ---------------------------------------------------------------------------------------------------------------------
 * © Commissariat a l'Energie Atomique et aux Energies Alternatives (CEA)
 * ---------------------------------------------------------------------------------------------------------------------
 * Contributors: Shebli Anvar (shebli.anvar@cea.fr)
 * ---------------------------------------------------------------------------------------------------------------------
 * This software is part of the Utilities software project.
 * ---------------------------------------------------------------------------------------------------------------------
 * FREE SOFTWARE LICENCING
 * This software is governed by the CeCILL license under French law and abiding  * by the rules of distribution of free
 * software. You can use, modify and/or redistribute the software under the terms of the CeCILL license as circulated by
 * CEA, CNRS and INRIA at the following URL: "http://www.cecill.info". As a counterpart to the access to the source code
 * and rights to copy, modify and redistribute granted by the license, users are provided only with a limited warranty
 * and the software's author, the holder of the economic rights, and the successive licensors have only limited
 * liability. In this respect, the user's attention is drawn to the risks associated with loading, using, modifying
 * and/or developing or reproducing the software by the user in light of its specific status of free software, that may
 * mean that it is complicated to manipulate, and that also therefore means that it is reserved for developers and
 * experienced professionals having in-depth computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling the security of their systems and/or data
 * to be ensured and, more generally, to use and operate it in the same conditions as regards security. The fact that
 * you are presently reading this means that you have had knowledge of the CeCILL license and that you accept its terms.
 * ---------------------------------------------------------------------------------------------------------------------
 * COMMERCIAL SOFTWARE LICENCING
 * You can obtain this software from CEA under other licencing terms for commercial purposes. For this you will need to
 * negotiate a specific contract with a legal representative of CEA.
 * =====================================================================================================================
 */

#ifndef utl_BitSet_h_INCLUDED
#define utl_BitSet_h_INCLUDED

#include "utl/RtBitSet.hpp"

namespace utl
{

/**
 * Template class representing a "word" made of an arbitrary number of bits set at compile time
 * @tparam S Indicates whether the bitset should be considered as a signed or unsigned word by taking on one of the 2
 *           enum values SIGNED or UNSIGNED.
 */
template < size_t NBITS, Signedness S >
class BitSet : public RtBitSet<S>
{
public:
	BitSet() : RtBitSet<S>(NBITS) {}

	template < typename W2 >
	BitSet(const W2& initWord) : RtBitSet<S>(NBITS, initWord) {}

	template < typename W2 >
	BitSet(const std::vector<W2>& initVect) : RtBitSet<S>(NBITS, initVect) {}

	template < Signedness S2 >
	BitSet(size_t _bitCount, const RtBitSet<S2>& initRtBitSet) : RtBitSet<S>(NBITS, initRtBitSet) {}

	static size_t bitCount () { return NBITS; }
	static size_t byteCount() { return atomCount() * RtBitSet<S>::atomByteCount(); }
	static size_t atomCount() { return NBITS%RtBitSet<S>::atomBitCount() == 0 ? NBITS/RtBitSet<S>::atomBitCount() : 1+NBITS/RtBitSet<S>::atomBitCount(); }
	static size_t minByteCount() { return bitCount()%CHAR_BIT == 0 ? bitCount()/CHAR_BIT : 1 + bitCount()/CHAR_BIT; }
};

} // namespace utl

template < size_t NBITS, ::utl::Signedness S >
std::ostream& operator << (std::ostream& os, const ::utl::BitSet<NBITS,S>& bitSet)
{
	return os << static_cast< ::utl::RtBitSet<S>& >(bitSet);
}

#endif /* utl_BitSet_h_INCLUDED */
